# Script   : place.tcl
# Version  : V4.4.4
# Function : "SAMPLE" SoC Encounter Placement & Optimization
# Date     : Mar.26th,2009
# Document : WEB-00188097, WEB-00298634(UX8L)
#
# SOCE version : 8.1
# Directory : sample_script/Layout_SOCE/place.tcl
#
# THIS LIBRARY IS FOR NEC ELECTRONICS INTERNAL USE ONLY.
# Copyright (C) NEC Electronics Corporation 2004-2009
#
#    VER 4.4.2 May-27-2008   BY T.Terayama
#        : Change TECHNOLOGY to UX8L
#    VER 4.4.3 Feb-27-2009   BY T.Terayama
#        : Change SoCE Version to 8.1
#        : Add deleteDanglingPort
#    VER 4.4.4 Mar-26-2009   BY T.Terayama
#        : add command for IO wiring
#        : comment out final preCTS STA
#        : change some parameter name

set TECHNOLOGY UX8L
set SOCE_VER 8.1
set ENCOUNTER_VERSION 8.1

# ============================================================================ #
# --- Data preparation --- #
# ============================================================================ #
set inputConfig        ../../database/conf/VDEC_soce.conf
set inputVerilog       ../01_floorplan/floorplan.v.gz
#set inputDef           ../02_proute/power.def.gz
set inputDef           ../01_floorplan/floorplan.def.gz
set inputLef           ../../database/lef/lef
#set inputSDC           ../../database/sdc/cpu8top_clock_200MHz.tcl
set inputSDC           ../../database/sdc/POCOP_TOP.sdc
set outputVerilog      ./pg.v.gz
set outputDef          ./pg.def.gz

#set clkSpec           " " 

#set inputECOFile       <input ECO file>
#set inputECOFile_DEL   <input ECO Delete file>
#set inputDelModule     <input Delete Module file>
#set Phy_Only_Cell      <Physical Only Cell List>

set maxLayerNumber     7

set encounterDBS	"encdbs"
set encounterRPT        "encrpt"

createSaveDir ${encounterDBS}
createSaveDir ${encounterRPT}

# ---------------------------------------------------------------------------- #
#  RC Factor
# ---------------------------------------------------------------------------- #
#  Caution : Please Calculate & Set Values Below
#            using Ostrich (Cadence tool)
#
# "SoC Encounter RC Factor Calibration Procedure Manual"
#                   (WEB-00091163)
# ---------------------------------------------------------------------------- #
set defCapMax  "1"
set detCapMax  "1"
set resMax     "1"

# ============================================================================ #
# --- Initial Setting --- #
# ============================================================================ #

if { $TECHNOLOGY == "CB12" || $TECHNOLOGY == "CB130M"|| $TECHNOLOGY == "CMOS12M" } {
        set DBU 100
} else {
        set DBU 1000
}

set numberOfPaths 100

set dbgUpdateDefRows 0
set dbgGPSAutoCellFunction 1

set dbgDefOutUnits $DBU
setImportMode -minDbuPerMicron $DBU
set preRouteSkipApplyGlobalNets 1
setVar dbgAllowPowerDomainMinGapZero 1
set sprUseRowOrient 1

set dbgIPOMaxBufDetourDist 100
set dpgBufInsMinTotCap 0

# ============================================================================ #
# --- Data in --- #
# ============================================================================ #
setImportMode -keepEmptyModule 1
setDoAssign on -new
generateTracks \
-m1HOffset 0 -m1HPitch 0.132 -m1VOffset 0.066 -m1VPitch 0.132 \
-m2HOffset 0 -m2HPitch 0.132 -m2VOffset 0.066 -m2VPitch 0.132 \
-m3HOffset 0 -m3HPitch 0.132 -m3VOffset 0.066 -m3VPitch 0.132 \
-m4HOffset 0 -m4HPitch 0.132 -m4VOffset 0.066 -m4VPitch 0.132 \
-m5HOffset 0 -m5HPitch 0.132 -m5VOffset 0.066 -m5VPitch 0.132 \
-m6HOffset 0 -m6HPitch 0.264 -m6VOffset 0.066 -m6VPitch 0.264 \
-m7HOffset 0 -m7HPitch 0.792 -m7VOffset 0.066 -m7VPitch 0.792

loadConfig                        $inputConfig 0
set rda_Input(ui_netlist)         $inputVerilog
set rda_Input(ui_leffile)         $inputLef
set rda_Input(ui_timingcon_file)  $inputSDC
commitConfig
if { [ info exists inputECOFile ] } { loadECO  $inputECOFile }
defIn                             $inputDef

setPreference ConstraintUserXOffset 0.066
setPreference BlockSnapRule 2
setPreference ConstraintUserXGrid 0.132
setPreference ConstraintUserYGrid 0.132
deselectAll
snapFPlan -block

checkMacroLLOnTrack -useM2M3Track

