# Script   : place.tcl
# Version  : V4.4.4
# Function : "SAMPLE" SoC Encounter Placement & Optimization
# Date     : Mar.26th,2009
# Document : WEB-00188097, WEB-00298634(UX8L)
#
# SOCE version : 8.1
# Directory : sample_script/Layout_SOCE/place.tcl
#
# THIS LIBRARY IS FOR NEC ELECTRONICS INTERNAL USE ONLY.
# Copyright (C) NEC Electronics Corporation 2004-2009
#
#    VER 4.4.2 May-27-2008   BY T.Terayama
#        : Change TECHNOLOGY to UX8L
#    VER 4.4.3 Feb-27-2009   BY T.Terayama
#        : Change SoCE Version to 8.1
#        : Add deleteDanglingPort
#    VER 4.4.4 Mar-26-2009   BY T.Terayama
#        : add command for IO wiring
#        : comment out final preCTS STA
#        : change some parameter name

set TECHNOLOGY UX8L
set SOCE_VER 8.1
set ENCOUNTER_VERSION 8.1

# ============================================================================ #
# --- Data preparation --- #
# ============================================================================ #
set inputConfig        ../../database/conf/VDEC_soce.conf
set inputVerilog       ../01_floorplan/floorplan.v.gz
#set inputDef           ../02_proute/power.def.gz
set inputDef           ../01_floorplan/floorplan.def.gz
set inputLef           ../../database/lef/lef
#set inputSDC           ../../database/sdc/cpu8top_clock_200MHz.tcl
set inputSDC           ../../database/sdc/POCOP_TOP.sdc
set outputVerilog      ./pg.v.gz
set outputDef          ./pg.def.gz

#set clkSpec           " " 

#set inputECOFile       <input ECO file>
#set inputECOFile_DEL   <input ECO Delete file>
#set inputDelModule     <input Delete Module file>
#set Phy_Only_Cell      <Physical Only Cell List>

set maxLayerNumber     7

set encounterDBS	"encdbs"
set encounterRPT        "encrpt"

createSaveDir ${encounterDBS}
createSaveDir ${encounterRPT}

# ---------------------------------------------------------------------------- #
#  RC Factor
# ---------------------------------------------------------------------------- #
#  Caution : Please Calculate & Set Values Below
#            using Ostrich (Cadence tool)
#
# "SoC Encounter RC Factor Calibration Procedure Manual"
#                   (WEB-00091163)
# ---------------------------------------------------------------------------- #
set defCapMax  "1"
set detCapMax  "1"
set resMax     "1"

# ============================================================================ #
# --- Initial Setting --- #
# ============================================================================ #

if { $TECHNOLOGY == "CB12" || $TECHNOLOGY == "CB130M"|| $TECHNOLOGY == "CMOS12M" } {
        set DBU 100
} else {
        set DBU 1000
}

set numberOfPaths 100

set dbgUpdateDefRows 0
set dbgGPSAutoCellFunction 1

set dbgDefOutUnits $DBU
setImportMode -minDbuPerMicron $DBU
set preRouteSkipApplyGlobalNets 1
setVar dbgAllowPowerDomainMinGapZero 1
set sprUseRowOrient 1

set dbgIPOMaxBufDetourDist 100
set dpgBufInsMinTotCap 0

# ============================================================================ #
# --- Data in --- #
# ============================================================================ #
setImportMode -keepEmptyModule 1
setDoAssign on -new
generateTracks \
-m1HOffset 0 -m1HPitch 0.132 -m1VOffset 0.066 -m1VPitch 0.132 \
-m2HOffset 0 -m2HPitch 0.132 -m2VOffset 0.066 -m2VPitch 0.132 \
-m3HOffset 0 -m3HPitch 0.132 -m3VOffset 0.066 -m3VPitch 0.132 \
-m4HOffset 0 -m4HPitch 0.132 -m4VOffset 0.066 -m4VPitch 0.132 \
-m5HOffset 0 -m5HPitch 0.132 -m5VOffset 0.066 -m5VPitch 0.132 \
-m6HOffset 0 -m6HPitch 0.264 -m6VOffset 0.066 -m6VPitch 0.264 \
-m7HOffset 0 -m7HPitch 0.792 -m7VOffset 0.066 -m7VPitch 0.792

loadConfig                        $inputConfig 0
set rda_Input(ui_netlist)         $inputVerilog
set rda_Input(ui_leffile)         $inputLef
set rda_Input(ui_timingcon_file)  $inputSDC
commitConfig
if { [ info exists inputECOFile ] } { loadECO  $inputECOFile }
defIn                             $inputDef

setPreference ConstraintUserXOffset 0.066
setPreference BlockSnapRule 2
setPreference ConstraintUserXGrid 0.132
setPreference ConstraintUserYGrid 0.132
deselectAll
snapFPlan -block

checkMacroLLOnTrack -useM2M3Track

clearGlobalNets
globalNetConnect VDD -type pgpin -pin VDD -inst * -verbose
globalNetConnect VDD -type tiehi -inst * -verbose
globalNetConnect VSS -type pgpin -pin VSS -inst * -verbose
globalNetConnect VSS -type tielo -inst * -verbose
# globalNetConnect VDE -type pgpin -pin VDE -inst * -verbose

cutRow
clearCutRow
deselectAll

set toplayer 8
set botlayer 1

setSnapGrid -layer { 1 }               -pitch 0.011 0.011
setSnapGrid -layer { 2 3 4 5 }         -pitch 0.033 0.033
setSnapGrid -layer { V12 V23 V34 V45 } -pitch 0.066 0.066

set v_spacing [expr 0.132 * 6]
set h_spacing [expr 0.132 * 6]
set vlayer M2L
set hlayer M3L
# set v_offset 0.627
# set h_offset 0.528
set v_offset [expr 0.132*200] 
set h_offset [expr 0.132*200]
set v_width 0.99
set h_width 0.99
#        -jog_distance [expr 0.132*15] \
#        -threshold [expr 0.132*2]\

addRing -nets { VDD VSS } \
        -follow core \
        -around core \
        -layer_top ${hlayer} \
        -layer_bottom ${hlayer} \
        -width_top ${h_width} \
        -width_bottom ${h_width} \
        -width_right ${v_width} \
        -width_left ${v_width} \
        -layer_right ${vlayer} \
        -layer_left ${vlayer} \
        -spacing_top ${h_spacing} \
        -spacing_bottom ${h_spacing} \
        -spacing_right ${v_spacing} \
        -spacing_left ${v_spacing} \
        -offset_top ${h_offset} \
        -offset_bottom ${h_offset} \
        -offset_right ${v_offset} \
        -offset_left ${v_offset} \
        -stacked_via_top_layer M6S \
        -stacked_via_bottom_layer M1L \
	-snap_wire_center_to_grid Grid \
	-tr 0 -bl 0 -br 0 -lt 0 -rt 0 -lb 0 -rb 0 
deselectAll

setAddStripeOption -remove_floating_stripe_over_block 0
addStripe -nets {VSS VDD } \
  -layer M6S \
    -direction vertical \
    -spacing [expr 0.264 * 20] \
    -width 0.99 \
    -set_to_set_distance [expr 0.264 * 60] \
    -xleft_offset 15.345 \
    -block_ring_top_layer_limit M1T \
    -block_ring_bottom_layer_limit M1L \
    -padcore_ring_top_layer_limit M1T \
    -padcore_ring_bottom_layer_limit M1L \
    -stacked_via_top_layer PM \
    -stacked_via_bottom_layer M1L \
    -merge_stripes_value [expr 0.264*5] \
	-snap_wire_center_to_grid Grid \
	-via_using_exact_crossover_size 1


#    -spacing [expr 0.264 * 20] \
#    -width [expr 0.264 * 4] \
#    -layer M4L \
#    -xleft_offset [expr 0.264 * 15] \


set connectlayer 4
set con_name VSS
sroute -nets { VSS } \
    -connect { padPin } \
    -padPinAllGeomsConnect \
    -padPinLayerRange { 1 5 } \
    -padPinWidth 0.99 \
    -split_long_via {0.264 0.264 0.264}

set connectlayer 4
set con_name VDD

sroute -nets { VDD } \
        -connect { padPin } \
    	-padPinAllGeomsConnect \
        -blockPinTarget { nearestTarget } \
        -padPinLayerRange { 3 5 } \
    	-padPinWidth 0.99 \
    -split_long_via {0.264 0.264 0.264}


srouteSecondaryPinAlighViaOnStripe true

sroute -nets { VSS VDD } \
    -connect { blockPin corePin floatingStripe } \
    -layerChangeRange { 1 8 } \
    -blockPinTarget { nearestRingStripe } \
    -padPinPortConnect { allPort oneGeom } \
    -area {-901.956 -901.956 901.956 901.956} \
    -targetViaTopLayer 8 \
    -targetViaBottomLayer 1 

#    -split_long_via {0.264 0.264 0.264}
#    -allowJogging 1 \
#    -allowLayerChange 1 \
#    -blockPin useLef \
#    -checkAlignedSecondaryPin 1 \
#    -crossoverViaBottomLayer 1 \
#    -crossoverViaTopLayer 8 \

saveNetlist pg.v.gz
defout -floorplan -placement -netlist -routing -noCutRow ${DESIGN}_pg.def.gz
