# Script   : place.tcl
# Version  : V4.4.4
# Function : "SAMPLE" SoC Encounter Placement & Optimization
# Date     : Mar.26th,2009
# Document : WEB-00188097, WEB-00298634(UX8L)
#
# SOCE version : 8.1
# Directory : sample_script/Layout_SOCE/place.tcl
#
# THIS LIBRARY IS FOR NEC ELECTRONICS INTERNAL USE ONLY.
# Copyright (C) NEC Electronics Corporation 2004-2009
#
#    VER 4.4.2 May-27-2008   BY T.Terayama
#        : Change TECHNOLOGY to UX8L
#    VER 4.4.3 Feb-27-2009   BY T.Terayama
#        : Change SoCE Version to 8.1
#        : Add deleteDanglingPort
#    VER 4.4.4 Mar-26-2009   BY T.Terayama
#        : add command for IO wiring
#        : comment out final preCTS STA
#        : change some parameter name

set TECHNOLOGY UX8L
set SOCE_VER 8.1
set ENCOUNTER_VERSION 8.1

# ============================================================================ #
# --- Data preparation --- #
# ============================================================================ #
set inputConfig        ../../database/conf/VDEC_soce.conf
set inputDef           ../02_proute/POCOP_TOP_pg.def.gz
set inputVerilog       ../02_proute/pg.v.gz
set inputLef           ../../database/lef/lef
#set inputSDC           ../../database/sdc/cpu8top_clock_200MHz.tcl
set inputSDC           ../../database/sdc/POCOP_TOP.sdc
set outputVerilog      ./place.v.gz
set outputDef          ./place.def.gz
set dontuse            ./file/dont_use.tcl

#set clkSpec           " " 

#set inputECOFile       <input ECO file>
#set inputECOFile_DEL   <input ECO Delete file>
#set inputDelModule     <input Delete Module file>
#set Phy_Only_Cell      <Physical Only Cell List>

set maxLayerNumber     7

set encounterDBS	"encdbs"
set encounterRPT        "encrpt"

createSaveDir ${encounterDBS}
createSaveDir ${encounterRPT}

# ---------------------------------------------------------------------------- #
#  RC Factor
# ---------------------------------------------------------------------------- #
#  Caution : Please Calculate & Set Values Below
#            using Ostrich (Cadence tool)
#
# "SoC Encounter RC Factor Calibration Procedure Manual"
#                   (WEB-00091163)
# ---------------------------------------------------------------------------- #
set defCapMax  "1"
set detCapMax  "1"
set resMax     "1"

# ============================================================================ #
# --- Initial Setting --- #
# ============================================================================ #

if { $TECHNOLOGY == "CB12" || $TECHNOLOGY == "CB130M"|| $TECHNOLOGY == "CMOS12M" } {
        set DBU 100
} else {
        set DBU 1000
}

set numberOfPaths 100

set dbgUpdateDefRows 0
set dbgGPSAutoCellFunction 1

set dbgDefOutUnits $DBU
setImportMode -minDbuPerMicron $DBU
set preRouteSkipApplyGlobalNets 1
setVar dbgAllowPowerDomainMinGapZero 1
set sprUseRowOrient 1

set dbgIPOMaxBufDetourDist 100
set dpgBufInsMinTotCap 0

# ============================================================================ #
# --- Data in --- #
# ============================================================================ #
setImportMode -keepEmptyModule 1
setDoAssign on -new

loadConfig                        $inputConfig 0
set rda_Input(ui_netlist)         $inputVerilog
set rda_Input(ui_leffile)         $inputLef
set rda_Input(ui_timingcon_file)  $inputSDC
commitConfig
if { [ info exists inputECOFile ] } { loadECO  $inputECOFile }
defIn                             $inputDef


source $dontuse

### for flat flow (from here) ###
#loadLefFile $inputLef
#loadDefNetlist $inputDef
#saveNetlist $inputDef.flat.v
#freeDesign
#loadConfig $inputConfig 0
#set rda_Input(ui_netlist) $inputDef.flat.v
#set rda_Input(ui_leffile) $inputLef
#set rda_Input(ui_timingcon_file) $inputSDC
#commitConfig
#if { [ info exists inputECOFile ] } { loadECO  $inputECOFile }
#defIn -verilog_from_def_netlist_flow $inputDef
### for flat flow (to here) ###

setCteReport
setOpCond {}

# ============================================================================ #
# --- Setting After Importing Design --- #
# ============================================================================ #

set designName [dbgDesignName]

if { $TECHNOLOGY == "CB65L" || $TECHNOLOGY == "UX7LS" } {
# ---------------------------------------------------------------------------- #
# Notice : Please use suitable "setPrerouteAsObs" setting
#          after reading the procedure manual of each 
#          individual function.
# ---------------------------------------------------------------------------- #
        setPrerouteAsObs {1}
        set dbgUseSiteAsIGU 1
        setPreference BlockSnapRule 2
}
if { $TECHNOLOGY == "UX8L" } {
# ---------------------------------------------------------------------------- #
# Notice : Please use suitable "setPrerouteAsObs" setting
#          after reading the procedure manual of each
#          individual function.
# ---------------------------------------------------------------------------- #
        setPrerouteAsObs {1}
        setPreference BlockSnapRule 2
}

# ---------------------------------------------------------------------------- #
# Notice : Setting below is only for placement of Hard Macros.
#          This setting is not available for hierarchical partitioning.
# ---------------------------------------------------------------------------- #
if { $TECHNOLOGY == "CB65L" } {
        setPreference UserGridUnit 1
        setPreference ConstraintUserXGrid 0.2
        setPreference ConstraintUserYGrid 0.2
        setPreference ConstraintUserXOffset 0.15
        setPreference ConstraintUserYOffset 0.0
} elseif { $TECHNOLOGY == "UX7LS" } {
        setPreference UserGridUnit 1
        setPreference ConstraintUserXGrid 0.18
        setPreference ConstraintUserYGrid 0.18
        setPreference ConstraintUserXOffset 0.0
        setPreference ConstraintUserYOffset 0.0
}

# ============================================================================ #
# --- RC extraction scaling factor --- #
# ============================================================================ #
setRCFactor -defcap $defCapMax -detcap $detCapMax -res $resMax

# ============================================================================ #
# --- specify cts file --- #
# ============================================================================ #
if { [ info var clkSpec ] != "" } {
	specifyClockTree -file $clkSpec
} else {
	Puts "WARNING : specifyClockTree not executed because no CTS Spec Files are specified."
}

# ============================================================================ #
# --- Setting for trialRoute/IPO --- #
# ============================================================================ #
setTrialRouteMode -highEffort true -maxRouteLayer $maxLayerNumber -handlePreroute true
setAnalysisMode -checkType setup
setOptMode -optimizeAssignNet true -optimizeConstantNet true -rebuffer true -bufferAssignNets true
setOptMode -minimizeArea true -setupTargetSlack 0.0 -preserveModuleFunction false -reclaimArea true

# ============================================================================ #
# --- gate-to-gate optimize ---
# ---------------------------------------------------------------------------- #
# The details are given in  "SoC Encounter Placement Function Procedure Manual"
#                                                               (WEB-00188097)
# ============================================================================ #
#source ./n2n_main.tcl

# ============================================================================ #
# Set Scan Reorder
#    -> set lockupLatch ( mux type )
# ============================================================================ #
# specifyLockupElement {-cell leaf_cellName | -inst <instName>} -in <ftname> -out <ftname>
# specifyScanChain <scanChainName> -start <ftname | instPinName> -stop <ftname | instPinName>
# scanTrace [-verbose|-lockup|-noLockup]

# ============================================================================ #
# --- placeDesign  ---
# ---------------------------------------------------------------------------- #
# - If you want to execute non-timingDriven placement,
#   please set -noTimingDriven instead of -timingDriven.
# - Please execute scan reordering after placement/optimization.
#   So setPlaceMode setting is -noReorderScan.
# - If placeDesign or OptDesign stop in the step of checkPlace related on Macro
#   pin, please set "setPlaceMode -allowBorderPinAbut true" 
# ============================================================================ #
setPlaceMode -timingDriven true -reorderScan false -maxRouteLayer $maxLayerNumber

if { [ info var clkSpec ] != "" } {
	setPlaceMode -clkGateAware true
} else {
	Puts "WARNING : -clkGateAware Option Not appended to setPlaceMode because no CTS Spec Files are specified."
}

##setPlaceMode -congEffort high
setPlaceMode -congEffort medium

# ---------------------------------------------------------------------------- #
# Notice : If your design has ERRORs in GateDRC (ERROR 0410 or 7030),
#          please Set setOptMode -neverAddPort
#          However, this option may be deteriorate the performance of
#          timing optimization
# ---------------------------------------------------------------------------- #
#
# setOptMode -neverAddPort

if { $TECHNOLOGY == "UX7LS" } {
	setPlaceMode -checkPinLayerForAccess 3
}

placeDesign
trialRoute -highEffort
saveDesign ${encounterDBS}/place.enc

##timeDesign -preCTS -outDir STA_placeDesign

# ============================================================================ #
# --- 1st preCTS IPO ---
# ============================================================================ #
setOptMode -fixFanoutLoad true -effort high -fixDrc true
setOptMode -setupTargetSlack 0.1

optDesign -preCTS -outDir ${encounterRPT}/preCTSIPO
saveDesign ${encounterDBS}/preCTSIPO.enc

timeDesign -preCTS -outDir ${encounterRPT}/STA_1st_preCTSIPO

# ============================================================================ #
# --- 2nd placeD + preCTSIPO ---
# ============================================================================ #
#setPlaceMode -NPSoftGuide true
#placeDesign -incremental
#saveDesign ${encounterDBS}/placeincr.enc
#
#setOptMode -fixFanoutLoad false
#setOptMode -setupTargetSlack 0.1
#optDesign -preCTS -outDir ${encounterRPT}/preCTSIPO2
#saveDesign ${encounterDBS}/preCTSIPO2.enc
#
#timeDesign -preCTS -outDir ${encounterRPT}/STA_2nd_preCTSIPO
#
# ============================================================================ #
# --- preCTSIPO -incremental ---
# ============================================================================ #
#setClockDomains -fromType register -toType output
#setOptMode -fixFanoutLoad false
#optDesign -preCTS -incr -outDir ${encounterRPT}/preCTSIPO2_INCR_R2O
#clearClockDomains
#  
#setClockDomains -fromType input -toType register
#setOptMode -fixFanoutLoad false
#optDesign -preCTS -incr -outDir ${encounterRPT}/preCTSIPO2_INCR_I2R
#clearClockDomains
#
#setClockDomains -fromType register -toType register
#setOptMode -criticalRange 0.3
#setOptMode -fixFanoutLoad false
#optDesign -preCTS -incr -outDir ${encounterRPT}/preCTSIPO2_INCR_R2R
#clearClockDomains
#
#saveDesign ${encounterDBS}/preCTSIPO2_INCR_R2R.enc
#
#timeDesign -preCTS -outDir ${encounterRPT}/STA_2nd_preCTSIPO_INCR_R2R
#
# ============================================================================ #
# --- Scan Reordering ---
# If your design has scan chain, please unlock the comment line below.
# ============================================================================ #
#scanTrace [-verbose|-lockup|-noLockup]
#scanReorder

# ============================================================================ #
# --- preCTS STA --- #
# ============================================================================ #
#timeDesign -preCTS -outDir ${encounterRPT}/STA_final

# ============================================================================ #
# --- SaveDesign --- #
# ============================================================================ #
if { $TECHNOLOGY == "CB65L" || $TECHNOLOGY == "UX7LS" || $TECHNOLOGY == "UX8L" } {
        set dbgLefDefOutVersion 5.6
} else {
        set dbgLefDefOutVersion 5.5
}

deleteDanglingPort

if { $TECHNOLOGY == "UX8L" } {
	globalNetConnect VDD -type pgpin -pin VDD
	globalNetConnect VSS -type pgpin -pin VSS
} else {
	globalNetConnect VDD -type pgpin -pin VDD
	globalNetConnect GND -type pgpin -pin GND
}

if { $TECHNOLOGY == "UX7LS" } {
	globalNetConnect GND -type tielo
	globalNetConnect VDD -type tiehi
}

if { $TECHNOLOGY == "UX8L" } {
        globalNetConnect VSS -type tielo
        globalNetConnect VDD -type tiehi
}

defOut -floorplan -placement -netlist \
        -noCutRow -routing $outputDef

### for flat flow (from here) ###
#defOut -verilog_from_def_netlist_flow -floorplan -placement -noCutRow -routing $outputDef
### for flat flow (to here) ###

if { [ info exists inputECOFile ] } {
  loadECO  $inputECOFile_DEL
  source   $inputDelModule
  saveNetlist -excludeCellInst $Phy_Only_Cell $outputVerilog
} else {
  saveNetlist $outputVerilog
}

# ============================================================================ #
# --- Exit --- #
# ============================================================================ #
# exit
# END
