# Script : cts.tcl
# Version : V1.4.2
# Function : Sample SoC Encounter CTS script
# Date : JUNE.9th,2009
# 
# Copyright (C) NEC Electronics Corporation 2005,2007-2009
#######################################################
#---------- Required setting by user -----------------
set inputConfig     ../../database/conf/VDEC_soce.conf
set inputDef        ../03_place/place.def.gz
set inputVerilog    ../03_place/place.v.gz
set inputLef        ../../database/lef/lef
#set inputSDC        ../../database/sdc/cpu8top_clock_200MHz.tcl
set inputSDC        ../../database/sdc/POCOP_TOP.sdc
set outputDef       ./cts.def.gz
set outputVerilog   ./cts.v.gz

#set ShieldNetList   ./file/shield_net.list

set clkSpec         ./file/cts_spec
set maxLayerNumber  7

#set inputECOFile       <input ECO file>
#set inputECOFile_DEL   <input ECO Delete file>
#set inputDelModule     <input Delete Module file>
#set Phy_Only_Cell      <Physical Only Cell List>

### NonDefaultRule for clocknets_upper ###
set UpperRouteNDR "SP"

### RC extraction scaling factor ###
set defCapMax 1
set detCapMax 1
set resMax    1

set saveDir FECTS_saveDIR
set clockNetOrg $saveDir/clocknets_org
set clockNet clocknets
set clockNetUpper clocknets_upper
set clockNetLeaf clocknets_leaf

##############################################################
#---------- define functions -----------------

proc get_clock_nets {specfile clknetfilename upperfilename leaffilename} {
  set fread [open $specfile r]
  set rlist [list ]
  set leaflist [list ]
  while {[gets $fread line] >= 0} {
    if { "AutoCTSRootPin" == [ lindex $line 0 ] } {
      lappend rlist [ lindex $line 1 ]
    }
  }
  close $fread

  set designname [dbCellName [dbgTopCell]]
  foreach clock $rlist {
    set leaflist [concat $leaflist [get_leaf_nets_sub $clock $designname.cts_trace]]
  }
  set cnlfile [open $leaffilename w]
  set cnfile [open $clknetfilename r]

  while { [gets $cnfile net] >= 0 } {
    regsub -all {\\} $net {} nobs_net
    set clocknet($nobs_net) $net
  }
  close $cnfile

  foreach net $leaflist {
    set netname [dbNetName $net]
    if { [info exists clocknet($netname)] == 0 } {
        continue
    }
    puts $cnlfile $clocknet($netname)
  }
  close $cnlfile

  set cnlfile [open $leaffilename r]
  set cnfile [open $clknetfilename r]
  set cnufile [open $upperfilename w]

  while { [gets $cnlfile net] >= 0 } {
    set clocklnet($net) $net
  }
  close $cnlfile
  while { [gets $cnfile net] >= 0 } {
    if {([info exists clocklnet($net)] != 0) || ([regexp (\#)+ $net])} {
        continue
    }
    puts $cnufile $net
  }
  close $cnufile
  close $cnfile

}

proc get_leaf_nets_sub {clock trace_file} {
  set trf [open $trace_file r]
  set net_list [list ]
  set clock_section_found 0
  while {([eof $trf] == 0) && ($clock_section_found == 0)} {
    set line [gets $trf]
    if {$line == "Tracing Clock $clock"} {
      set clock_section_found 1
    }
  }
  if {$clock_section_found == 0} {
    puts "clock section not found for clock $clock"
    return
  }
  set clock_section_found 0
  while {([eof $trf] == 0) && ($clock_section_found == 0)} {
    set line [gets $trf]
    if {[regexp "\\(Sync\\)(.*)" $line a hpin_name]} {
      if {[regexp (.*)/(.*) $hpin_name a inst_name pin_name]} {
        set termPtr [dbGetTermByName $inst_name $pin_name]
        set net [dbTermNet $termPtr]
      } else {
        set FTermPtr [dbGetFTermByName [dbgTopCell] $hpin_name]
        set net [dbFTermNet $FTermPtr]
      }
      if {[lsearch $net_list $net] < 0} {
        lappend net_list $net
      }
    } else {
      if {[string match "Tracing Clock*" $line]} {
        set clock_section_found 1
      }
    }
  }
  close $trf
  return $net_list
}

proc userGenBlockageForEachMacro {} { 
  global saveDir
  set design [dbgDesignName]
  Puts "\n\n\nInfo: Checking for hard macros in $design...\n"
  set cnt_inst 0
  set f [open $saveDir/routeblk.tcl w]
  dbForEachCellInst [dbgTopCell] instPtr {
    if { $instPtr != "0x0" } {
      set obj_type [dbObjType $instPtr]
      if { $obj_type == "dbcObjInst" } {
        set instName [dbObjName $instPtr]
        set cellName [dbInstCellName $instPtr]
        set cellPtr [dbInstCell $instPtr]
        if { [dbIsCellBlock $cellPtr] == 1 } {
            incr cnt_inst
            set instBox [dbInstBox $instPtr]
            set x1 [expr [lindex $instBox 0]*[dbHeadMicronPerDBU]]
            set y1 [expr [lindex $instBox 1]*[dbHeadMicronPerDBU]]
            set x2 [expr [lindex $instBox 2]*[dbHeadMicronPerDBU]]
            set y2 [expr [lindex $instBox 3]*[dbHeadMicronPerDBU]]
  
            ### Generate routing blockage for block instance ###
            Puts "\nInfo: Generating blockage for instance $instName ($cellName)..." 
            puts $f "# [dbInstCellName $instPtr] : [dbInstName $instPtr]"
            puts $f "createRouteBlk -box  $x1 $y1 $x2 $y2 -layer all -name BlkforCTS"
        }
      }
    }
  }
  Puts "\nInfo: Created RouteBlk Command for $cnt_inst macros in $saveDir/routeblk.tcl.\n"
  close $f
}

##############################################################

#####################################
# input design
#####################################

#setDoAssign -buffer [buf name] on

set DBU 1000
set dbgLefDefOutVersion 5.6

set dbgUpdateDefRows 0
set dbgUseNewFootPrint  1

set dbgGPSAutoCellFunction 1
set dbgDefOutUnits $DBU
setImportMode -minDbuPerMicron $DBU
set preRouteSkipApplyGlobalNets 1
setVar dbgAllowPowerDomainMinGapZero 1
set sprUseRowOrient 1
set dbgIPOMaxBufDetourDist 100
set dpgBufInsMinTotCap 0

### The following Option is for CB65L/UX7LS ###
#set dbgUseSiteAsIGU 1
### The following Option is for CB65L/UX7LS/UX8L ###
setPreference UserGridUnit 1

### for general flow (from here) ###
loadConfig                        $inputConfig 0
set rda_Input(ui_netlist)         $inputVerilog
set rda_Input(ui_leffile)         $inputLef
set rda_Input(ui_timingcon_file)  $inputSDC
commitConfig
if { [ info exists inputECOFile ] } { loadECO  $inputECOFile }
defIn                             $inputDef
### for general flow (to here) ###

### for flat flow (from here) ###
#loadLefFile $inputLef
#loadDefNetlist $inputDef
#saveNetlist $inputDef.flat.v
#freeDesign
#loadConfig $inputConfig 0
#set rda_Input(ui_netlist) $inputDef.flat.v
#set rda_Input(ui_leffile) $inputLef
#set rda_Input(ui_timingcon_file) $inputSDC
#commitConfig
#if { [ info exists inputECOFile ] } { loadECO  $inputECOFile }
#defIn -verilog_from_def_netlist_flow $inputDef
### for flat flow (to here) ###

### RC extraction scaling factor ###
setRCFactor -defcap $defCapMax -detcap $detCapMax -res $resMax

createSaveDir $saveDir

#### Use for creating routeblockage on macros.
#userGenBlockageForEachMacro
#source $saveDir/routeblk.tcl
#setCTSMode -routeBlockagePadding true 
#setCTSMode -paddingInPitch 4

#setPrerouteAsObs {1}

setTrialRouteMode -maxRouteLayer $maxLayerNumber
setTrialRouteMode -handlePreroute true
setCTSMode -honorPreRoute true
setCTSMode -skipTrackInTrialRoute 1
setCTSMode -bypassGuideNetNrTerm -1 -bypassGuideUnderLength -1
setCTSMode -useHVRC true
setCTSMode -routeBlockageHV true
#setCTSMode -moveGate false
setCTSMode -synthNewCluster true
#setCTSMode -maxGatedCell 500000
setCTSMode -guideEndEdgeLengthLimit 20

specifyClockTree -file $clkSpec
checkUnique

#### replace root cell FCTS -> BUF
setOptMode -rPlace
set_dont_touch LDH_CTS false

#set_dont_touch core/clk_blk/cts_clk1/CTS_ROOT false
#set_dont_touch core/clk_blk/cts_clk2/CTS_ROOT false
set_dont_touch IBUF_RST_N false

#dbSetInstPlacementStatus [dbGetInstByName "core/clk_blk/cts_clk1/CTS_ROOT"] dbcPlaced
#ecoChangeCell -inst "core/clk_blk/cts_clk1/CTS_ROOT" -cell "LDH_BUF_S_10"

#dbSetInstPlacementStatus [dbGetInstByName "core/clk_blk/cts_clk2/CTS_ROOT"] dbcPlaced
#ecoChangeCell -inst "core/clk_blk/cts_clk2/CTS_ROOT" -cell "LDH_BUF_S_10"

dbSetInstPlacementStatus [dbGetInstByName "IBUF_RST_N"] dbcPlaced
ecoChangeCell -inst "IBUF_RST_N" -cell "LDH_BUF_S_10"

#set_dont_touch core/clk_blk/cts_clk1/CTS_ROOT true 
#set_dont_touch core/clk_blk/cts_clk2/CTS_ROOT true
set_dont_touch IBUF_RST_N true

#####################################
# CTS
#####################################
###
#source file/makeHFNCTSScript.tcl
#makeHFNCTSScript 20 LDH_BUF_S_10
#specifyClockTree -file cpu8top.hf_cts_mod
### for UX7LS/UX8L Power-EM aware CTS (SoCE8.1) ###
ckSynthesis -rguide .routing_guide.rgf \
            -report $saveDir/cts.ctsrpt \
            -forceReconvergent
checkClockTreeCellHalo -file halo_check.rpt

globalNetConnect VDD -type pgpin -pin VDD
#globalNetConnect GND -type pgpin -pin GND
globalNetConnect VSS -type pgpin -pin VSS

### The following Option is for UX7LS/UX8L ###
#globalNetConnect GND -type tielo
globalNetConnect VSS -type tielo
globalNetConnect VDD -type tiehi

#####################################
# Create Clock nets files
#####################################
### for general flow (from here) ###
saveClockNets -def -output $clockNetOrg
### for general flow (to here) ###

### for flat flow (from here) ###
#saveClockNets -output $clockNetOrg
### for flat flow (to here) ###

system sed -e '/^#/d' -e 's/clk_net //g' $clockNetOrg > $clockNet

Puts "\nInfo: Create $clockNetLeaf and $clockNetUpper\n"
get_clock_nets $clkSpec $clockNet $clockNetUpper $clockNetLeaf

#####################################
# Add  NONDEFAULTRULE
#####################################
##source vias_ux7ls_sample.tcl
set netFName $clockNetUpper 
set NETFILE [open $netFName]
while { [eof $NETFILE] == 0 } {
  set g [gets  $NETFILE]
  if { $g == "" } { continue }
  setAttribute \
    -net $g \
   -non_default_rule $UpperRouteNDR

##  editChangeRule \
##    -net $g \
##    -from SP \
##    -to default
##
##  editChangeVia  -net $g -from ${FromVias} -to ${ToVias} -signal_only 1

}
close $NETFILE
#----------------------
# shield_net_ibpo
#---------------------
#insertRepeater -postRoute -check -rule file/ibpo.rule -selNet $ShieldNetList

#####################################
# output Netlist/DEF
#####################################
### for general flow (from here) ###
defOut -floorplan -placement -netlist -routing -noCutRow $outputDef
### for general flow (to here) ###

### for flat flow (from here) ###
#defOut -verilog_from_def_netlist_flow -floorplan -placement -netlist -routing -noCutRow $outputDef
### for flat flow (to here) ###

if { [ info exists inputECOFile ] } {
  loadECO  $inputECOFile_DEL
  source   $inputDelModule
  saveNetlist -excludeCellInst $Phy_Only_Cell $outputVerilog
} else {
  saveNetlist $outputVerilog
}

# exit
