# Script   : cdfill_soce.tcl
# Version  : Ver3.7.0
# Function : "SAMPLE" SoC Encounter Decoupling Cell Insertion
# Date     : Sep.24,2009
#
# SOCE version : 08.10-e296
#
# THIS LIBRARY IS FOR NEC ELECTRONICS INTERNAL USE ONLY.
# Copyright (C) NEC Electronics Corporation 2009
#
#
#    Ver3.7.0 Sep-24-2009
#

set ENCOUNTER_VERSION 8.1
set SOCE_VER 08.10-e296

set TECHNOLOGY   UX8L
#set DESIGN   <design name> ; # Specify in the config file

set inputConfig  ../../database/conf/VDEC_soce.conf
set inputVerilog ../04_cts/cts.v.gz
# set inputDef     ../07_metal_fill/metalfill.def.gz
set inputDef     ../06_shield/shield.def.gz
set inputLef     ../../database/lef/lef
set inputSDC     ../../database/sdc/POCOP_TOP.sdc

set outputDef    ./decap.def.gz

set max_nr_iter 10 ; # number of iteration
set NumOfCPU     1 ; # number of CPU(s)

set encounterDBS "encdbs"

set targetCap           "100000000"
set decap_prefix        "FILL_G_"
set normal_prefix       "FILL_F_"
set rptDeCapFinal       "final"

set maxLayerNumber 7

set defineScript   ./file/define_cdfill_9Grid_GC1.tcl

#set inputECOFile   <input ECO file>

#****************************
# Initial Setting
#****************************

if { $TECHNOLOGY == "CB12" || $TECHNOLOGY == "CB130M"|| $TECHNOLOGY == "CMOS12M" } {
        set DBU 100
} else {
        set DBU 1000
}

set numberOfPaths 100

if { $ENCOUNTER_VERSION == "4.2" } {
        set dbgDefOutPhysicalPin 1
}
set dbgUpdateDefRows 0
set dbgGPSAutoCellFunction 1

set dbgDefOutUnits $DBU
setImportMode -minDbuPerMicron $DBU
set preRouteSkipApplyGlobalNets 1
setVar dbgAllowPowerDomainMinGapZero 1
set sprUseRowOrient 1

set dbgIPOMaxBufDetourDist 100
set dpgBufInsMinTotCap 0

set vgApiUseMT 1
setMultiCpuUsage -numThreads  $NumOfCPU

#****************************
# Defining Procedures
#****************************

source ./cdfill_count_rpt.tcl

proc cntCell { cellName } {
        set cnt 0
        set cellPtr [dbgTopCell]
        dbForEachCellInst $cellPtr instPtr {
                set instName [dbInstName $instPtr]
                set leafCell [dbInstCellName $instPtr]
                if { $leafCell == $cellName } {
                        incr cnt
                }
        }
        Puts "$cellName : $cnt"
}

#****************************
# Importing Design
#****************************

### for general flow (from here) ###
loadConfig $inputConfig 0
#setUIVar rda_Input ui_topcell  "$DESIGN"
setUIVar rda_Input ui_netlist "$inputVerilog"
setUIVar rda_Input ui_timingcon_file "$inputSDC"
setUIVar rda_Input ui_leffile "$inputLef"
commitConfig

if { [ info exists inputECOFile ] } { loadECO  $inputECOFile }

defIn $inputDef
### for general flow (to here) ###

### for flat flow (from here) ###
#loadLefFile $inputLef
#loadDefNetlist $inputDef
#saveNetlist $inputDef.flat.v
#freeDesign
#loadConfig $inputConfig 0
#setUIVar rda_Input ui_netlist $inputDef.flat.v
#commitConfig
#defIn -verilog_from_def_netlist_flow $inputDef
### for flat flow (to here) ###

if { $TECHNOLOGY == "CB65L" || $TECHNOLOGY == "UX7LS" || $TECHNOLOGY == "UX8L" || $TECHNOLOGY == "UX8LD" } {
# ---------------------------------------------------------------------------- #
# Notice : Please use suitable "setPrerouteAsObs" setting
# after reading the procedure manual of each
# individual function.
# ---------------------------------------------------------------------------- #
	setPrerouteAsObs { }
	set dbgUseSiteAsIGU 1
	setPreference BlockSnapRule 2
}

if { $TECHNOLOGY == "CB65L" } {
	setPreference UserGridUnit 1
	setPreference ConstraintUserXGrid 0.2
	setPreference ConstraintUserYGrid 0.2
	setPreference ConstraintUserXOffset 0.15
	setPreference ConstraintUserYOffset 0.0
} elseif { $TECHNOLOGY == "UX7LS" } {
	setPreference UserGridUnit 0
	setPreference ConstraintUserXGrid 0.18
	setPreference ConstraintUserYGrid 1.62
	setPreference ConstraintUserXOffset 0.0
	setPreference ConstraintUserYOffset 0.18
}

#****************************
# Setting Values
#****************************

#-----------------------------#
# Deifine Values for addDeCap #
#-----------------------------#
source $defineScript
#-----------------------------#

set designName [dbgDesignName]

set dbgIPOMaxBufDetourDist 100
set dpgBufInsMinTotCap 0

setTrialRouteMode -maxRouteLayer $maxLayerNumber
setPlaceMode -maxRouteLayer $maxLayerNumber

#****************************
# globalNetConnect
#****************************

globalNetConnect VDD -type pgpin -pin VDD
globalNetConnect VSS -type pgpin -pin VSS
globalNetConnect VDD -type tiehi
globalNetConnect VSS -type tielo

#****************************
# Inserting DeCap Cells
#****************************

rptDeCap initial

createSaveDir deCapLog
addDeCap -totCap $targetCap -maxNrIter $max_nr_iter -addDRCCheckOption {-merge} -addDRCCheckOption {-minHole} -log deCapLog/addDeCap.log -prefix $decap_prefix

rptDeCap add_decap

# =========================================== #
# If your design has multiple power domains,
# please use -powerdomain option for addDeCap.
# =========================================== #
#source CB65L/define_cdfill_TG3.tcl
#set power_domain VDD
#set decap_prefix FILL_G_VDD
#set normal_prefix FILL_F_VDD
#
#addDeCap -totCap $targetCap -log deCapLog/addDeCap.log \
#          -prefix $decap_prefix -powerDomain $power_domain
#
#rptDeCap add_decap_$power_domain
#
#clearDeCapCellCandidates

#****************************
# Inserting Normal FillCells
#****************************

addFiller -cell $normal_fill -prefix $normal_prefix -noDRC 

# =========================================== #
# If your design has multiple power domains,
# please use -powerdomain option for addFiller.
# =========================================== #
#foreach fillCell $normal_fill {
#        addFiller -cell $fillCell -prefix $normal_prefix \
#                  -noDRC -powerDomain $power_domain
#}
#

# =========================================== #
# If your design has multiple power domains,
# please source all deCap cell definition scripts
# used for addDeCap before rptDeCap final
# =========================================== #
#
#source <define_cdfill_TG3.tcl>
#source <define_cdfill_TG5.tcl>
#source <define_cdfill_TG7.tcl>

rptDeCap final


#****************************
# Saving Design
#****************************

if { $TECHNOLOGY == "CB65L" || $TECHNOLOGY == "UX7LS" || $TECHNOLOGY == "UX8L" || $TECHNOLOGY == "UX8LD" } {
        set dbgLefDefOutVersion 5.6
} else {
        set dbgLefDefOutVersion 5.5
}


if { $TECHNOLOGY == "UX8L" || $TECHNOLOGY == "UX8LD" } {

  globalNetConnect VDD -type pgpin -pin VDD
  globalNetConnect VSS -type pgpin -pin VSS
  globalNetConnect VDD -type tiehi
  globalNetConnect VSS -type tielo

} else {

  globalNetConnect VDD -type pgpin -pin VDD
  globalNetConnect GND -type pgpin -pin GND

  if { $TECHNOLOGY == "UX7LS" } {
    globalNetConnect GND -type tielo
    globalNetConnect VDD -type tiehi
  }

}

if { $ENCOUNTER_VERSION == "5.2" } {
        defOut -physicalPin -floorplan -placement -netlist -noCutRow -routing $outputDef
} else {
### for general flow (from here) ###
        defOut -floorplan -placement -netlist -noCutRow -routing $outputDef
### for general flow (to here) ###

### for flat flow (from here) ###
#defOut -verilog_from_def_netlist_flow -floorplan -placement -netlist -noCutRow -routing $outputDef
### for flat flow (to here) ###
}

exit
# END
