#!/usr/local/bin/perl -w
#
# cdfill_summary.pl -- Show decoupling-cell summary
#   Copyright (C) NEC Electronics Corporation 2004,2005,2006,2007,2008,2009
#
# Note:
#   - This script is for CB90L (V1.1.0 or later), CB90M (V2.2.0 or later),
#     and CB90H, CB130M, CB65L, UX7LS and UX8L.
#   - Only standard decoupling cells are supported.
# History:
#   2009-03-13 Ver1.9.0

use strict;

######## constants ########

my $VERSION = '1.7.0';
my $MODULE_DATE = '2008-03-27';
my $VERSION_MESSAGE = "cdfill_summary.pl V${VERSION} (${MODULE_DATE})";

my @REPORT_PHASE_LIST = ( 'initial', 'final', 'unknown' );
my @ANALYSIS_PHASE_LIST = ( 'final' );
  
 
######## main ########
 
print $VERSION_MESSAGE, "\n\n";
 
my $program_name = $0;
my $max_poly_dens = 100;
my $ignore_normal_fillcell = 0;
my $verbose = 0; 
my $tech_name ; 
my %CELL_TABLE = ();

get_option();

# If you use non-standard Cd cells, please add them in the table.
if ($tech_name =~ /CB130M/) {

%CELL_TABLE = (
  ######## CB130M ########
  #### CB130M standard decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TC4FILL1024GC1' => { sites => 1024, cap =>   3552.02, poly =>  0.00,  leak =>  66.18, footp => 'CB130M-C-1024'  },
  'TC4FILL512GC1'  => { sites =>  512, cap =>   1776.01, poly =>  0.00,  leak =>  33.09, footp => 'CB130M-C-0512'  },
  'TC4FILL256GC1'  => { sites =>  256, cap =>    888.10, poly =>  0.00,  leak =>  16.54, footp => 'CB130M-C-0256'  },
  'TC4FILL128GC1'  => { sites =>  128, cap =>    444.00, poly =>  0.00,  leak =>   8.27, footp => 'CB130M-C-0128'  },
  'TC4FILL64GC1'   => { sites =>   64, cap =>    222.01, poly =>  0.00,  leak =>   4.14, footp => 'CB130M-C-0064'  },
  'TC4FILL32GC1'   => { sites =>   32, cap =>    111.00, poly =>  0.00,  leak =>   2.07, footp => 'CB130M-C-0032'  },
  'TC4FILL16GC1'   => { sites =>   16, cap =>     55.50, poly =>  0.00,  leak =>   1.03, footp => 'CB130M-C-0016'  },
  'TC4FILL8GC1'    => { sites =>    8, cap =>     27.75, poly =>  0.00,  leak =>   0.52, footp => 'CB130M-C-0008'  },
  'TC4FILL4GC1'    => { sites =>    8, cap =>      7.19, poly =>  0.00,  leak =>   0.27, footp => 'CB130M-C-0008'  },
  #### CB130M normal fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TC4FILL1024'    => { sites => 1024, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB130M-N-1024'  },
  'TC4FILL512'     => { sites =>  512, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB130M-N-0512'  },
  'TC4FILL256'     => { sites =>  256, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB130M-N-0256'  },
  'TC4FILL128'     => { sites =>  128, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB130M-N-0128'  },
  'TC4FILL64'      => { sites =>   64, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB130M-N-0064'  },
  'TC4FILL32'      => { sites =>   32, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB130M-N-0032'  },
  'TC4FILL16'      => { sites =>   16, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB130M-N-0016'  },
  'TC4FILL8'       => { sites =>    8, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB130M1-N-0008'  },
  'TC4FILL4'       => { sites =>    4, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB130M1-N-0004'  },
  'TC4FILL2'       => { sites =>    2, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB130M1-N-0002'  },
  'TC4FILL1'       => { sites =>    1, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB130M1-N-0001'  }
);


} elsif ($tech_name =~ /CB90L/) {

%CELL_TABLE = (
  ######## CB90L V1 ########
  #### CB90L V1 LT standard decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TD3FILL1024GC1' => { sites => 1024, cap =>   1798.63, poly => 35.54,  leak =>  44.60, footp => 'CB90L1-C-1024'  },
  'TD3FILL512GC1'  => { sites =>  512, cap =>    897.74, poly => 35.63,  leak =>  22.30, footp => 'CB90L1-C-0512'  },
  'TD3FILL256GC1'  => { sites =>  256, cap =>    447.29, poly => 35.79,  leak =>  11.10, footp => 'CB90L1-C-0256'  },
  'TD3FILL128GC1'  => { sites =>  128, cap =>    222.07, poly => 36.13,  leak =>   5.55, footp => 'CB90L1-C-0128'  },
  'TD3FILL64GC1'   => { sites =>   64, cap =>    109.46, poly => 36.81,  leak =>   2.76, footp => 'CB90L1-C-0064'  },
  'TD3FILL32GC1'   => { sites =>   32, cap =>     53.15, poly => 38.17,  leak =>   1.37, footp => 'CB90L1-C-0032'  },
  'TD3FILL16GC1'   => { sites =>   16, cap =>     25.00, poly => 40.88,  leak =>   0.67, footp => 'CB90L1-C-0016'  },
  'TD3FILL8GC1'    => { sites =>    8, cap =>     12.62, poly => 41.77,  leak =>   0.19, footp => 'CB90L1-C-0008'  },
  #### CB90L V1 LT normal fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TD3FILL1024'    => { sites => 1024, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90L1-N-1024'  },
  'TD3FILL512'     => { sites =>  512, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90L1-N-0512'  },
  'TD3FILL256'     => { sites =>  256, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90L1-N-0256'  },
  'TD3FILL128'     => { sites =>  128, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90L1-N-0128'  },
  'TD3FILL64'      => { sites =>   64, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90L1-N-0064'  },
  'TD3FILL32'      => { sites =>   32, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90L1-N-0032'  },
  'TD3FILL16'      => { sites =>   16, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90L1-N-0016'  },
  'TD3FILL8'       => { sites =>    8, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90L1-N-0008'  },
  'TD3FILL4'       => { sites =>    4, cap =>         0, poly => 24.04,  leak =>      0, footp => 'CB90L1-N-0004'  },
  'TD3FILL2'       => { sites =>    2, cap =>         0, poly => 24.04,  leak =>      0, footp => 'CB90L1-N-0002'  },
  'TD3FILL1'       => { sites =>    1, cap =>         0, poly => 30.05,  leak =>      0, footp => 'CB90L1-N-0001'  },
  ######## CB90L V2 ########
  #### CB90L V2 LT standard decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TD4FILL1024GC1' => { sites => 1024, cap =>   1798.63, poly => 35.54,  leak =>  44.60, footp => 'CB90L2-C-1024'  },
  'TD4FILL512GC1'  => { sites =>  512, cap =>    897.74, poly => 35.63,  leak =>  22.30, footp => 'CB90L2-C-0512'  },
  'TD4FILL256GC1'  => { sites =>  256, cap =>    447.29, poly => 35.79,  leak =>  11.10, footp => 'CB90L2-C-0256'  },
  'TD4FILL128GC1'  => { sites =>  128, cap =>    222.07, poly => 36.13,  leak =>   5.55, footp => 'CB90L2-C-0128'  },
  'TD4FILL64GC1'   => { sites =>   64, cap =>    109.46, poly => 36.81,  leak =>   2.76, footp => 'CB90L2-C-0064'  },
  'TD4FILL32GC1'   => { sites =>   32, cap =>     53.15, poly => 38.17,  leak =>   1.37, footp => 'CB90L2-C-0032'  },
  'TD4FILL16GC1'   => { sites =>   16, cap =>     25.00, poly => 40.88,  leak =>   0.67, footp => 'CB90L2-C-0016'  },
  'TD4FILL8GC1'    => { sites =>    8, cap =>     12.62, poly => 41.77,  leak =>   0.19, footp => 'CB90L2-C-0008'  },
  #### CB90L V2 LT normal fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TD4FILL1024'    => { sites => 1024, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90L2-N-1024'  },
  'TD4FILL512'     => { sites =>  512, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90L2-N-0512'  },
  'TD4FILL256'     => { sites =>  256, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90L2-N-0256'  },
  'TD4FILL128'     => { sites =>  128, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90L2-N-0128'  },
  'TD4FILL64'      => { sites =>   64, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90L2-N-0064'  },
  'TD4FILL32'      => { sites =>   32, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90L2-N-0032'  },
  'TD4FILL16'      => { sites =>   16, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90L2-N-0016'  },
  'TD4FILL8'       => { sites =>    8, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90L2-N-0008'  },
  'TD4FILL4'       => { sites =>    4, cap =>         0, poly => 24.04,  leak =>      0, footp => 'CB90L2-N-0004'  },
  'TD4FILL2'       => { sites =>    2, cap =>         0, poly => 24.04,  leak =>      0, footp => 'CB90L2-N-0002'  },
  'TD4FILL1'       => { sites =>    1, cap =>         0, poly => 30.05,  leak =>      0, footp => 'CB90L2-N-0001'  },
);


} elsif ($tech_name =~ /CB90M/) {

%CELL_TABLE = (
  ######## CB90M V2 ########
  #### CB90M V2 M9 standard decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TC2FILL1024GC1' => { sites => 1024, cap =>   2030.72, poly => 35.58,  leak => 893.40, footp => 'CB90M2-C-1024' },
  'TC2FILL512GC1'  => { sites =>  512, cap =>   1013.58, poly => 35.66,  leak => 445.90, footp => 'CB90M2-C-0512' },
  'TC2FILL256GC1'  => { sites =>  256, cap =>    505.01, poly => 35.83,  leak => 222.20, footp => 'CB90M2-C-0256' },
  'TC2FILL128GC1'  => { sites =>  128, cap =>    250.72, poly => 36.17,  leak => 110.30, footp => 'CB90M2-C-0128' },
  'TC2FILL64GC1'   => { sites =>   64, cap =>    123.58, poly => 36.85,  leak =>  54.34, footp => 'CB90M2-C-0064' },
  'TC2FILL32GC1'   => { sites =>   32, cap =>     60.01, poly => 38.21,  leak =>  26.37, footp => 'CB90M2-C-0032' },
  'TC2FILL16GC1'   => { sites =>   16, cap =>     28.22, poly => 40.92,  leak =>  12.38, footp => 'CB90M2-C-0016' },
  'TC2FILL8GC1'    => { sites =>    8, cap =>     14.58, poly => 41.77,  leak =>   3.74, footp => 'CB90M2-C-0008' },
  'TC2FILL4GC1'    => { sites =>    4, cap =>      3.33, poly => 26.66,  leak =>   1.36, footp => 'CB90M2-C-0004' },
  #### CB90M V2 M9 low density decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TC2FILL1024GC2' => { sites => 1024, cap =>    903.17, poly => 26.47,  leak => 396.20, footp => 'CB90M2-C-1024' },
  'TC2FILL512GC2'  => { sites =>  512, cap =>    451.58, poly => 26.47,  leak => 198.10, footp => 'CB90M2-C-0512' },
  'TC2FILL256GC2'  => { sites =>  256, cap =>    225.79, poly => 26.47,  leak =>  99.05, footp => 'CB90M2-C-0256' },
  'TC2FILL128GC2'  => { sites =>  128, cap =>    112.90, poly => 26.47,  leak =>  49.53, footp => 'CB90M2-C-0128' },
  'TC2FILL64GC2'   => { sites =>   64, cap =>     56.45, poly => 26.47,  leak =>  24.76, footp => 'CB90M2-C-0064' },
  'TC2FILL32GC2'   => { sites =>   32, cap =>     28.22, poly => 26.47,  leak =>  12.38, footp => 'CB90M2-C-0032' },
  'TC2FILL16GC2'   => { sites =>   16, cap =>     14.11, poly => 26.47,  leak =>   6.19, footp => 'CB90M2-C-0016' },
  'TC2FILL8GC2'    => { sites =>    8, cap =>      6.96, poly => 26.23,  leak =>   2.85, footp => 'CB90M2-C-0008' },
  #### CB90M V2 M9 normal fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TC2FILL1024'    => { sites => 1024, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90M2-N-1024' },
  'TC2FILL512'     => { sites =>  512, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90M2-N-0512' },
  'TC2FILL256'     => { sites =>  256, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90M2-N-0256' },
  'TC2FILL128'     => { sites =>  128, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90M2-N-0128' },
  'TC2FILL64'      => { sites =>   64, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90M2-N-0064' },
  'TC2FILL32'      => { sites =>   32, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90M2-N-0032' },
  'TC2FILL16'      => { sites =>   16, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90M2-N-0016' },
  'TC2FILL8'       => { sites =>    8, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90M2-N-0008' },
  'TC2FILL4'       => { sites =>    4, cap =>         0, poly => 24.04,  leak =>      0, footp => 'CB90M2-N-0004' },
  'TC2FILL2'       => { sites =>    2, cap =>         0, poly => 24.04,  leak =>      0, footp => 'CB90M2-N-0002' },
  'TC2FILL1'       => { sites =>    1, cap =>         0, poly => 30.05,  leak =>      0, footp => 'CB90M2-N-0001' },
  ######## CB90M V3 ########
  #### CB90M V3 M9 standard decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TC4FILL1024GC1' => { sites => 1024, cap =>   2030.72, poly => 35.58,  leak => 893.40, footp => 'CB90M3-C-1024' },
  'TC4FILL512GC1'  => { sites =>  512, cap =>   1013.58, poly => 35.66,  leak => 445.90, footp => 'CB90M3-C-0512' },
  'TC4FILL256GC1'  => { sites =>  256, cap =>    505.01, poly => 35.83,  leak => 222.20, footp => 'CB90M3-C-0256' },
  'TC4FILL128GC1'  => { sites =>  128, cap =>    250.72, poly => 36.17,  leak => 110.30, footp => 'CB90M3-C-0128' },
  'TC4FILL64GC1'   => { sites =>   64, cap =>    123.58, poly => 36.85,  leak =>  54.34, footp => 'CB90M3-C-0064' },
  'TC4FILL32GC1'   => { sites =>   32, cap =>     60.01, poly => 38.21,  leak =>  26.37, footp => 'CB90M3-C-0032' },
  'TC4FILL16GC1'   => { sites =>   16, cap =>     28.22, poly => 40.92,  leak =>  12.38, footp => 'CB90M3-C-0016' },
  'TC4FILL8GC1'    => { sites =>    8, cap =>     14.58, poly => 41.77,  leak =>   3.74, footp => 'CB90M3-C-0008' },
  'TC4FILL4GC1'    => { sites =>    4, cap =>      3.33, poly => 26.66,  leak =>   1.36, footp => 'CB90M3-C-0004' },
  #### CB90M V3 M9 low density decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TC4FILL1024GC2' => { sites => 1024, cap =>    903.17, poly => 26.47,  leak => 396.20, footp => 'CB90M3-C-1024' },
  'TC4FILL512GC2'  => { sites =>  512, cap =>    451.58, poly => 26.47,  leak => 198.10, footp => 'CB90M3-C-0512' },
  'TC4FILL256GC2'  => { sites =>  256, cap =>    225.79, poly => 26.47,  leak =>  99.05, footp => 'CB90M3-C-0256' },
  'TC4FILL128GC2'  => { sites =>  128, cap =>    112.90, poly => 26.47,  leak =>  49.53, footp => 'CB90M3-C-0128' },
  'TC4FILL64GC2'   => { sites =>   64, cap =>     56.45, poly => 26.47,  leak =>  24.76, footp => 'CB90M3-C-0064' },
  'TC4FILL32GC2'   => { sites =>   32, cap =>     28.22, poly => 26.47,  leak =>  12.38, footp => 'CB90M3-C-0032' },
  'TC4FILL16GC2'   => { sites =>   16, cap =>     14.11, poly => 26.47,  leak =>   6.19, footp => 'CB90M3-C-0016' },
  'TC4FILL8GC2'    => { sites =>    8, cap =>      6.96, poly => 26.23,  leak =>   2.85, footp => 'CB90M3-C-0008' },
  #### CB90M V3 M9 normal fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TC4FILL1024'    => { sites => 1024, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90M3-N-1024' },
  'TC4FILL512'     => { sites =>  512, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90M3-N-0512' },
  'TC4FILL256'     => { sites =>  256, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90M3-N-0256' },
  'TC4FILL128'     => { sites =>  128, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90M3-N-0128' },
  'TC4FILL64'      => { sites =>   64, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90M3-N-0064' },
  'TC4FILL32'      => { sites =>   32, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90M3-N-0032' },
  'TC4FILL16'      => { sites =>   16, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90M3-N-0016' },
  'TC4FILL8'       => { sites =>    8, cap =>         0, poly => 12.02,  leak =>      0, footp => 'CB90M3-N-0008' },
  'TC4FILL4'       => { sites =>    4, cap =>         0, poly => 24.04,  leak =>      0, footp => 'CB90M3-N-0004' },
  'TC4FILL2'       => { sites =>    2, cap =>         0, poly => 24.04,  leak =>      0, footp => 'CB90M3-N-0002' },
  'TC4FILL1'       => { sites =>    1, cap =>         0, poly => 30.05,  leak =>      0, footp => 'CB90M3-N-0001' },
);


} elsif ($tech_name =~ /CB90H/) {

%CELL_TABLE = (
  ######## CB90H ########
  #### CB90H H12 standard decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TB1FILL1024GC1' => { sites => 1024, cap => 22.36*128, poly => 42.59,  leak => 862.72, footp => 'CB90H-C-1024'  },
  'TB1FILL512GC1'  => { sites =>  512, cap => 22.36* 64, poly => 42.59,  leak => 431.36, footp => 'CB90H-C-0512'  },
  'TB1FILL256GC1'  => { sites =>  256, cap => 22.36* 32, poly => 42.59,  leak => 215.68, footp => 'CB90H-C-0256'  },
  'TB1FILL128GC1'  => { sites =>  128, cap => 22.36* 16, poly => 42.59,  leak => 107.84, footp => 'CB90H-C-0128'  },
  'TB1FILL64GC1',  => { sites =>   64, cap => 22.36*  8, poly => 42.59,  leak =>  53.92, footp => 'CB90H-C-0064'  },
  'TB1FILL32GC1',  => { sites =>   32, cap => 22.36*  4, poly => 42.59,  leak =>  26.96, footp => 'CB90H-C-0032'  },
  'TB1FILL16GC1',  => { sites =>   16, cap => 22.36*  2, poly => 42.59,  leak =>  13.48, footp => 'CB90H-C-0016'  },
  'TB1FILL8GC1',   => { sites =>    8, cap => 22.36    , poly => 42.59,  leak =>   6.74, footp => 'CB90H-C-0008'  },
  'TB1FILL6GC1',   => { sites =>    6, cap => 17.29    , poly => 45.47,  leak =>   5.16, footp => 'CB90H-C-0006'  },
  #### CB90H H12 normal fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TB1FILL1024'    => { sites => 1024, cap =>         0, poly => 12.59,  leak =>      0, footp => 'CB90H-N-1024'  },
  'TB1FILL512'     => { sites =>  512, cap =>         0, poly => 12.59,  leak =>      0, footp => 'CB90H-N-0512'  },
  'TB1FILL256'     => { sites =>  256, cap =>         0, poly => 12.59,  leak =>      0, footp => 'CB90H-N-0256'  },
  'TB1FILL128'     => { sites =>  128, cap =>         0, poly => 12.59,  leak =>      0, footp => 'CB90H-N-0128'  },
  'TB1FILL64'      => { sites =>   64, cap =>         0, poly => 12.59,  leak =>      0, footp => 'CB90H-N-0064'  },
  'TB1FILL32'      => { sites =>   32, cap =>         0, poly => 12.59,  leak =>      0, footp => 'CB90H-N-0032'  },
  'TB1FILL16'      => { sites =>   16, cap =>         0, poly => 12.59,  leak =>      0, footp => 'CB90H-N-0016'  },
  'TB1FILL8'       => { sites =>    8, cap =>         0, poly => 12.59,  leak =>      0, footp => 'CB90H-N-0008'  },
  'TB1FILL6'       => { sites =>    6, cap =>         0, poly => 16.78,  leak =>      0, footp => 'CB90H-N-0006'  },
  'TB1FILL4'       => { sites =>    4, cap =>         0, poly => 25.17,  leak =>      0, footp => 'CB90H-N-0004'  },
  'TB1FILL2'       => { sites =>    2, cap =>         0, poly => 25.17,  leak =>      0, footp => 'CB90H-N-0002'  },
  'TB1FILL1'       => { sites =>    1, cap =>         0, poly => 31.46,  leak =>      0, footp => 'CB90H-N-0001'  },
);

} elsif ($tech_name =~ /CB65L/) {
%CELL_TABLE = (
  ######## CB65L (Common Area) (1.2V/5GHz/125deg) ########
  #### CB65L(Common) (1.2V/5GHz/125deg) standard decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TG3FILL1024GC1' => { sites => 1024, cap =>   1135.62, poly => 29.40,  leak =>  11.87, footp => 'CB65LTG3-C-1024'  },
  'TG3FILL512GC1'  => { sites =>  512, cap =>    568.82, poly => 29.43,  leak =>   5.95, footp => 'CB65LTG3-C-0512'  },
  'TG3FILL256GC1'  => { sites =>  256, cap =>    285.42, poly => 29.49,  leak =>   2.99, footp => 'CB65LTG3-C-0256'  },
  'TG3FILL128GC1'  => { sites =>  128, cap =>    143.72, poly => 29.60,  leak =>   1.51, footp => 'CB65LTG3-C-0128'  },
  'TG3FILL64GC1'   => { sites =>   64, cap =>     72.87, poly => 29.83,  leak =>   0.77, footp => 'CB65LTG3-C-0064'  },
  'TG3FILL32GC1'   => { sites =>   32, cap =>     38.99, poly => 31.28,  leak =>   0.42, footp => 'CB65LTG3-C-0032'  },
  'TG3FILL16GC1'   => { sites =>   16, cap =>     19.71, poly => 32.07,  leak =>   0.22, footp => 'CB65LTG3-C-0016'  },
  'TG3FILL8GC1'    => { sites =>    8, cap =>      2.96, poly => 19.85,  leak =>   0.01, footp => 'CB65LTG3-C-0008'  },
  #### CB65L(Common) (1.2V/5GHz/125deg) low leak decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TG3FILL1024GC3' => { sites => 1024, cap =>   1076.64, poly => 29.40,  leak =>   1.31, footp => 'CB65LTG3-C-1024'  },
  'TG3FILL512GC3'  => { sites =>  512, cap =>    539.20, poly => 29.43,  leak =>   0.66, footp => 'CB65LTG3-C-0512'  },
  'TG3FILL256GC3'  => { sites =>  256, cap =>    270.48, poly => 29.49,  leak =>   0.33, footp => 'CB65LTG3-C-0256'  },
  'TG3FILL128GC3'  => { sites =>  128, cap =>    136.12, poly => 29.60,  leak =>   0.17, footp => 'CB65LTG3-C-0128'  },
  'TG3FILL64GC3'   => { sites =>   64, cap =>     68.94, poly => 29.83,  leak =>   0.08, footp => 'CB65LTG3-C-0064'  },
  'TG3FILL32GC3'   => { sites =>   32, cap =>     36.63, poly => 31.28,  leak =>   0.05, footp => 'CB65LTG3-C-0032'  },
  'TG3FILL16GC3'   => { sites =>   16, cap =>     18.00, poly => 32.07,  leak =>   0.02, footp => 'CB65LTG3-C-0016'  },
  #### CB65L(Common) (1.2V/5GHz/125deg) normal fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TG3FILL1024'    => { sites => 1024, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-1024'  },
  'TG3FILL512'     => { sites =>  512, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0512'  },
  'TG3FILL256'     => { sites =>  256, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0256'  },
  'TG3FILL128'     => { sites =>  128, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0128'  },
  'TG3FILL64'      => { sites =>   64, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0064'  },
  'TG3FILL32'      => { sites =>   32, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0032'  },
  'TG3FILL16'      => { sites =>   16, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0016'  },
  'TG3FILL8'       => { sites =>    8, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0008'  },
  'TG3FILL4'       => { sites =>    4, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0004'  },
  'TG3FILL2'       => { sites =>    2, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0002'  },
  'TG3FILL1'       => { sites =>    1, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0001'  },
  ######## CB65L (Back Bias Area) (1.2V/5GHz/125deg) ########
  #### CB65L(Back Bias) (1.2V/5GHz/125deg) standard decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TG5FILL1024GC1' => { sites => 1024, cap =>   1135.62, poly => 29.40,  leak =>  11.87, footp => 'CB65LTG5-C-1024'  },
  'TG5FILL512GC1'  => { sites =>  512, cap =>    568.82, poly => 29.43,  leak =>   5.95, footp => 'CB65LTG5-C-0512'  },
  'TG5FILL256GC1'  => { sites =>  256, cap =>    285.42, poly => 29.49,  leak =>   2.99, footp => 'CB65LTG5-C-0256'  },
  'TG5FILL128GC1'  => { sites =>  128, cap =>    143.72, poly => 29.60,  leak =>   1.51, footp => 'CB65LTG5-C-0128'  },
  'TG5FILL64GC1'   => { sites =>   64, cap =>     72.87, poly => 29.83,  leak =>   0.77, footp => 'CB65LTG5-C-0064'  },
  'TG5FILL32GC1'   => { sites =>   32, cap =>     38.99, poly => 31.28,  leak =>   0.42, footp => 'CB65LTG5-C-0032'  },
  'TG5FILL16GC1'   => { sites =>   16, cap =>     19.71, poly => 32.07,  leak =>   0.22, footp => 'CB65LTG5-C-0016'  },
  'TG5FILL8GC1'    => { sites =>    8, cap =>      2.96, poly => 19.85,  leak =>   0.01, footp => 'CB65LTG5-C-0008'  },
  #### CB65L(Back Bias) (1.2V/5GHz/125deg) low leak decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TG5FILL1024GC3' => { sites => 1024, cap =>   1076.64, poly => 29.40,  leak =>   1.31, footp => 'CB65LTG5-C-1024'  },
  'TG5FILL512GC3'  => { sites =>  512, cap =>    539.20, poly => 29.43,  leak =>   0.66, footp => 'CB65LTG5-C-0512'  },
  'TG5FILL256GC3'  => { sites =>  256, cap =>    270.48, poly => 29.49,  leak =>   0.33, footp => 'CB65LTG5-C-0256'  },
  'TG5FILL128GC3'  => { sites =>  128, cap =>    136.12, poly => 29.60,  leak =>   0.17, footp => 'CB65LTG5-C-0128'  },
  'TG5FILL64GC3'   => { sites =>   64, cap =>     68.94, poly => 29.83,  leak =>   0.08, footp => 'CB65LTG5-C-0064'  },
  'TG5FILL32GC3'   => { sites =>   32, cap =>     36.63, poly => 31.28,  leak =>   0.05, footp => 'CB65LTG5-C-0032'  },
  'TG5FILL16GC3'   => { sites =>   16, cap =>     18.00, poly => 32.07,  leak =>   0.02, footp => 'CB65LTG5-C-0016'  },
  #### CB65L(Back Bias) (1.2V/5GHz/125deg) normal fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TG5FILL1024'    => { sites => 1024, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-1024'  },
  'TG5FILL512'     => { sites =>  512, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0512'  },
  'TG5FILL256'     => { sites =>  256, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0256'  },
  'TG5FILL128'     => { sites =>  128, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0128'  },
  'TG5FILL64'      => { sites =>   64, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0064'  },
  'TG5FILL32'      => { sites =>   32, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0032'  },
  'TG5FILL16'      => { sites =>   16, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0016'  },
  'TG5FILL8'       => { sites =>    8, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0008'  },
  'TG5FILL4'       => { sites =>    4, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0004'  },
  'TG5FILL2'       => { sites =>    2, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0002'  },
  'TG5FILL1'       => { sites =>    1, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0001'  },
  ######## CB65L (Power Control Area) (1.2V/5GHz/125deg) ########
  #### CB65L(Power Control) (1.2V/5GHz/125deg) standard decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TG7FILL1024GC1' => { sites => 1024, cap =>   1135.62, poly => 29.40,  leak =>  11.87, footp => 'CB65LTG7-C-1024'  },
  'TG7FILL512GC1'  => { sites =>  512, cap =>    568.82, poly => 29.43,  leak =>   5.95, footp => 'CB65LTG7-C-0512'  },
  'TG7FILL256GC1'  => { sites =>  256, cap =>    285.42, poly => 29.49,  leak =>   2.99, footp => 'CB65LTG7-C-0256'  },
  'TG7FILL128GC1'  => { sites =>  128, cap =>    143.72, poly => 29.60,  leak =>   1.51, footp => 'CB65LTG7-C-0128'  },
  'TG7FILL64GC1'   => { sites =>   64, cap =>     72.87, poly => 29.83,  leak =>   0.77, footp => 'CB65LTG7-C-0064'  },
  'TG7FILL32GC1'   => { sites =>   32, cap =>     38.99, poly => 31.28,  leak =>   0.42, footp => 'CB65LTG7-C-0032'  },
  'TG7FILL16GC1'   => { sites =>   16, cap =>     19.71, poly => 32.07,  leak =>   0.22, footp => 'CB65LTG7-C-0016'  },
  'TG7FILL8GC1'    => { sites =>    8, cap =>      2.96, poly => 19.85,  leak =>   0.01, footp => 'CB65LTG7-C-0008'  },
  #### CB65L(Power Control) (1.2V/5GHz/125deg) low leak decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TG7FILL1024GC3' => { sites => 1024, cap =>   1076.64, poly => 29.40,  leak =>   1.31, footp => 'CB65LTG7-C-1024'  },
  'TG7FILL512GC3'  => { sites =>  512, cap =>    539.20, poly => 29.43,  leak =>   0.66, footp => 'CB65LTG7-C-0512'  },
  'TG7FILL256GC3'  => { sites =>  256, cap =>    270.48, poly => 29.49,  leak =>   0.33, footp => 'CB65LTG7-C-0256'  },
  'TG7FILL128GC3'  => { sites =>  128, cap =>    136.12, poly => 29.60,  leak =>   0.17, footp => 'CB65LTG7-C-0128'  },
  'TG7FILL64GC3'   => { sites =>   64, cap =>     68.94, poly => 29.83,  leak =>   0.08, footp => 'CB65LTG7-C-0064'  },
  'TG7FILL32GC3'   => { sites =>   32, cap =>     36.63, poly => 31.28,  leak =>   0.05, footp => 'CB65LTG7-C-0032'  },
  'TG7FILL16GC3'   => { sites =>   16, cap =>     18.00, poly => 32.07,  leak =>   0.02, footp => 'CB65LTG7-C-0016'  },
  #### CB65L(Power Control) (1.2V/5GHz/125deg) normal fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TG7FILL1024'    => { sites => 1024, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-1024'  },
  'TG7FILL512'     => { sites =>  512, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0512'  },
  'TG7FILL256'     => { sites =>  256, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0256'  },
  'TG7FILL128'     => { sites =>  128, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0128'  },
  'TG7FILL64'      => { sites =>   64, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0064'  },
  'TG7FILL32'      => { sites =>   32, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0032'  },
  'TG7FILL16'      => { sites =>   16, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0016'  },
  'TG7FILL8'       => { sites =>    8, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0008'  },
  'TG7FILL4'       => { sites =>    4, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0004'  },
  'TG7FILL2'       => { sites =>    2, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0002'  },
  'TG7FILL1'       => { sites =>    1, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0001'  },
# ========================================== #
# If you use 0.74V for Power Supply,
# Please Comment Out Lines Above and
# Delete Comment Lines Below.
# ========================================== #
#  ######## CB65L (Common Area) (0.74V/5GHz/125deg) ########
#  #### CB65L(Common) (0.74V/5GHz/125deg) standard decoupling fillcell ####
#  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
#  'TG3FILL1024GC1' => { sites => 1024, cap =>   1056.65, poly => 29.40,  leak =>  0.997, footp => 'CB65LTG3-C-1024'  },
#  'TG3FILL512GC1'  => { sites =>  512, cap =>    529.29, poly => 29.43,  leak =>  0.500, footp => 'CB65LTG3-C-0512'  },
#  'TG3FILL256GC1'  => { sites =>  256, cap =>    265.61, poly => 29.49,  leak =>  0.251, footp => 'CB65LTG3-C-0256'  },
#  'TG3FILL128GC1'  => { sites =>  128, cap =>    133.77, poly => 29.60,  leak =>  0.127, footp => 'CB65LTG3-C-0128'  },
#  'TG3FILL64GC1'   => { sites =>   64, cap =>     67.85, poly => 29.83,  leak =>  0.065, footp => 'CB65LTG3-C-0064'  },
#  'TG3FILL32GC1'   => { sites =>   32, cap =>     36.18, poly => 31.28,  leak =>  0.035, footp => 'CB65LTG3-C-0032'  },
#  'TG3FILL16GC1'   => { sites =>   16, cap =>     18.27, poly => 32.07,  leak =>  0.018, footp => 'CB65LTG3-C-0016'  },
#  'TG3FILL8GC1'    => { sites =>    8, cap =>      2.77, poly => 19.85,  leak =>  0.001, footp => 'CB65LTG3-C-0008'  },
#  #### CB65L(Common) (0.74V/5GHz/125deg) low leak decoupling fillcell ####
#  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
#  'TG3FILL1024GC3' => { sites => 1024, cap =>    948.88, poly => 29.40,  leak =>  0.189, footp => 'CB65LTG3-C-1024'  },
#  'TG3FILL512GC3'  => { sites =>  512, cap =>    475.04, poly => 29.43,  leak =>  0.094, footp => 'CB65LTG3-C-0512'  },
#  'TG3FILL256GC3'  => { sites =>  256, cap =>    238.12, poly => 29.49,  leak =>  0.047, footp => 'CB65LTG3-C-0256'  },
#  'TG3FILL128GC3'  => { sites =>  128, cap =>    119.66, poly => 29.60,  leak =>  0.024, footp => 'CB65LTG3-C-0128'  },
#  'TG3FILL64GC3'   => { sites =>   64, cap =>     60.43, poly => 29.83,  leak =>  0.012, footp => 'CB65LTG3-C-0064'  },
#  'TG3FILL32GC3'   => { sites =>   32, cap =>     31.69, poly => 31.28,  leak =>  0.006, footp => 'CB65LTG3-C-0032'  },
#  'TG3FILL16GC3'   => { sites =>   16, cap =>     14.18, poly => 32.07,  leak =>  0.003, footp => 'CB65LTG3-C-0016'  },
#  #### CB65L(Common) (0.74V/5GHz/125deg) normal fillcell ####
#  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
#  'TG3FILL1024'    => { sites => 1024, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-1024'  },
#  'TG3FILL512'     => { sites =>  512, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0512'  },
#  'TG3FILL256'     => { sites =>  256, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0256'  },
#  'TG3FILL128'     => { sites =>  128, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0128'  },
#  'TG3FILL64'      => { sites =>   64, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0064'  },
#  'TG3FILL32'      => { sites =>   32, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0032'  },
#  'TG3FILL16'      => { sites =>   16, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0016'  },
#  'TG3FILL8'       => { sites =>    8, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0008'  },
#  'TG3FILL4'       => { sites =>    4, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0004'  },
#  'TG3FILL2'       => { sites =>    2, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0002'  },
#  'TG3FILL1'       => { sites =>    1, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG3-N-0001'  },
#  ######## CB65L (Back Bias Area) (0.74V/5GHz/125deg) ########
#  #### CB65L(Back Bias) (0.74V/5GHz/125deg) standard decoupling fillcell ####
#  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
#  'TG5FILL1024GC1' => { sites => 1024, cap =>   1056.65, poly => 29.40,  leak =>  0.997, footp => 'CB65LTG5-C-1024'  },
#  'TG5FILL512GC1'  => { sites =>  512, cap =>    529.29, poly => 29.43,  leak =>  0.500, footp => 'CB65LTG5-C-0512'  },
#  'TG5FILL256GC1'  => { sites =>  256, cap =>    265.61, poly => 29.49,  leak =>  0.251, footp => 'CB65LTG5-C-0256'  },
#  'TG5FILL128GC1'  => { sites =>  128, cap =>    133.77, poly => 29.60,  leak =>  0.127, footp => 'CB65LTG5-C-0128'  },
#  'TG5FILL64GC1'   => { sites =>   64, cap =>     67.85, poly => 29.83,  leak =>  0.065, footp => 'CB65LTG5-C-0064'  },
#  'TG5FILL32GC1'   => { sites =>   32, cap =>     36.18, poly => 31.28,  leak =>  0.035, footp => 'CB65LTG5-C-0032'  },
#  'TG5FILL16GC1'   => { sites =>   16, cap =>     18.27, poly => 32.07,  leak =>  0.018, footp => 'CB65LTG5-C-0016'  },
#  'TG5FILL8GC1'    => { sites =>    8, cap =>      2.77, poly => 19.85,  leak =>  0.001, footp => 'CB65LTG5-C-0008'  },
#  #### CB65L(Back Bias) (0.74V/5GHz/125deg) low leak decoupling fillcell ####
#  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
#  'TG5FILL1024GC3' => { sites => 1024, cap =>    948.88, poly => 29.40,  leak =>  0.189, footp => 'CB65LTG5-C-1024'  },
#  'TG5FILL512GC3'  => { sites =>  512, cap =>    475.04, poly => 29.43,  leak =>  0.094, footp => 'CB65LTG5-C-0512'  },
#  'TG5FILL256GC3'  => { sites =>  256, cap =>    238.12, poly => 29.49,  leak =>  0.047, footp => 'CB65LTG5-C-0256'  },
#  'TG5FILL128GC3'  => { sites =>  128, cap =>    119.66, poly => 29.60,  leak =>  0.024, footp => 'CB65LTG5-C-0128'  },
#  'TG5FILL64GC3'   => { sites =>   64, cap =>     60.43, poly => 29.83,  leak =>  0.012, footp => 'CB65LTG5-C-0064'  },
#  'TG5FILL32GC3'   => { sites =>   32, cap =>     31.69, poly => 31.28,  leak =>  0.006, footp => 'CB65LTG5-C-0032'  },
#  'TG5FILL16GC3'   => { sites =>   16, cap =>     14.18, poly => 32.07,  leak =>  0.003, footp => 'CB65LTG5-C-0016'  },
#  #### CB65L(Back Bias) (0.74V/5GHz/125deg) normal fillcell ####
#  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
#  'TG5FILL1024'    => { sites => 1024, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-1024'  },
#  'TG5FILL512'     => { sites =>  512, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0512'  },
#  'TG5FILL256'     => { sites =>  256, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0256'  },
#  'TG5FILL128'     => { sites =>  128, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0128'  },
#  'TG5FILL64'      => { sites =>   64, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0064'  },
#  'TG5FILL32'      => { sites =>   32, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0032'  },
#  'TG5FILL16'      => { sites =>   16, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0016'  },
#  'TG5FILL8'       => { sites =>    8, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0008'  },
#  'TG5FILL4'       => { sites =>    4, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0004'  },
#  'TG5FILL2'       => { sites =>    2, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0002'  },
#  'TG5FILL1'       => { sites =>    1, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG5-N-0001'  },
#  ######## CB65L (Power Control Area) (0.74V/5GHz/125deg) ########
#  #### CB65L(Power Control) (0.74V/5GHz/125deg) standard decoupling fillcell ####
#  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
#  'TG7FILL1024GC1' => { sites => 1024, cap =>   1056.65, poly => 29.40,  leak =>  0.997, footp => 'CB65LTG7-C-1024'  },
#  'TG7FILL512GC1'  => { sites =>  512, cap =>    529.29, poly => 29.43,  leak =>  0.500, footp => 'CB65LTG7-C-0512'  },
#  'TG7FILL256GC1'  => { sites =>  256, cap =>    265.61, poly => 29.49,  leak =>  0.251, footp => 'CB65LTG7-C-0256'  },
#  'TG7FILL128GC1'  => { sites =>  128, cap =>    133.77, poly => 29.60,  leak =>  0.127, footp => 'CB65LTG7-C-0128'  },
#  'TG7FILL64GC1'   => { sites =>   64, cap =>     67.85, poly => 29.83,  leak =>  0.065, footp => 'CB65LTG7-C-0064'  },
#  'TG7FILL32GC1'   => { sites =>   32, cap =>     36.18, poly => 31.28,  leak =>  0.035, footp => 'CB65LTG7-C-0032'  },
#  'TG7FILL16GC1'   => { sites =>   16, cap =>     18.27, poly => 32.07,  leak =>  0.018, footp => 'CB65LTG7-C-0016'  },
#  'TG7FILL8GC1'    => { sites =>    8, cap =>      2.77, poly => 19.85,  leak =>  0.001, footp => 'CB65LTG7-C-0008'  },
#  #### CB65L(Power Control) (0.74V/5GHz/125deg) low leak decoupling fillcell ####
#  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
#  'TG7FILL1024GC3' => { sites => 1024, cap =>    948.88, poly => 29.40,  leak =>  0.189, footp => 'CB65LTG7-C-1024'  },
#  'TG7FILL512GC3'  => { sites =>  512, cap =>    475.04, poly => 29.43,  leak =>  0.094, footp => 'CB65LTG7-C-0512'  },
#  'TG7FILL256GC3'  => { sites =>  256, cap =>    238.12, poly => 29.49,  leak =>  0.047, footp => 'CB65LTG7-C-0256'  },
#  'TG7FILL128GC3'  => { sites =>  128, cap =>    119.66, poly => 29.60,  leak =>  0.024, footp => 'CB65LTG7-C-0128'  },
#  'TG7FILL64GC3'   => { sites =>   64, cap =>     60.43, poly => 29.83,  leak =>  0.012, footp => 'CB65LTG7-C-0064'  },
#  'TG7FILL32GC3'   => { sites =>   32, cap =>     31.69, poly => 31.28,  leak =>  0.006, footp => 'CB65LTG7-C-0032'  },
#  'TG7FILL16GC3'   => { sites =>   16, cap =>     14.18, poly => 32.07,  leak =>  0.003, footp => 'CB65LTG7-C-0016'  },
#  #### CB65L(Power Control) (0.74V/5GHz/125deg) normal fillcell ####
#  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
#  'TG7FILL1024'    => { sites => 1024, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-1024'  },
#  'TG7FILL512'     => { sites =>  512, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0512'  },
#  'TG7FILL256'     => { sites =>  256, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0256'  },
#  'TG7FILL128'     => { sites =>  128, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0128'  },
#  'TG7FILL64'      => { sites =>   64, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0064'  },
#  'TG7FILL32'      => { sites =>   32, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0032'  },
#  'TG7FILL16'      => { sites =>   16, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0016'  },
#  'TG7FILL8'       => { sites =>    8, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0008'  },
#  'TG7FILL4'       => { sites =>    4, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0004'  },
#  'TG7FILL2'       => { sites =>    2, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0002'  },
#  'TG7FILL1'       => { sites =>    1, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'CB65LTG7-N-0001'  },
);

} elsif ($tech_name =~ /UX7LS/) {
%CELL_TABLE = (
  ######## UX7LS(9Grid) ########
  #### UX7LS(9Grid) standard decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TH3FILL1024GC3' => { sites => 1024, cap =>    912.00, poly => 30.70,  leak =>  14.00, footp => 'UX7LSTH3-C-1024'  },
  'TH3FILL512GC3'  => { sites =>  512, cap =>    456.00, poly => 30.65,  leak =>  7.002, footp => 'UX7LSTH3-C-0512'  },
  'TH3FILL256GC3'  => { sites =>  256, cap =>    228.00, poly => 30.56,  leak =>  3.501, footp => 'UX7LSTH3-C-0256'  },
  'TH3FILL128GC3'  => { sites =>  128, cap =>    114.00, poly => 30.39,  leak =>  1.751, footp => 'UX7LSTH3-C-0128'  },
  'TH3FILL64GC3'   => { sites =>   64, cap =>     57.00, poly => 30.03,  leak =>  0.875, footp => 'UX7LSTH3-C-0064'  },
  'TH3FILL32GC3'   => { sites =>   32, cap =>     28.50, poly => 29.51,  leak =>  0.438, footp => 'UX7LSTH3-C-0032'  },
  'TH3FILL16GC3'   => { sites =>   16, cap =>     13.79, poly => 27.40,  leak =>  0.213, footp => 'UX7LSTH3-C-0016'  },
  'TH3FILL8GC3'    => { sites =>    8, cap =>      5.03, poly => 22.45,  leak =>  0.079, footp => 'UX7LSTH3-C-0008'  },
  #### UX7LS(9Grid) normal fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TH3FILL1024'    => { sites => 1024, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH3-N-1024'  },
  'TH3FILL512'     => { sites =>  512, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH3-N-0512'  },
  'TH3FILL256'     => { sites =>  256, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH3-N-0256'  },
  'TH3FILL128'     => { sites =>  128, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH3-N-0128'  },
  'TH3FILL64'      => { sites =>   64, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH3-N-0064'  },
  'TH3FILL32'      => { sites =>   32, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH3-N-0032'  },
  'TH3FILL16'      => { sites =>   16, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH3-N-0016'  },
  'TH3FILL8'       => { sites =>    8, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH3-N-0008'  },
  'TH3FILL4'       => { sites =>    4, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH3-N-0004'  },
  'TH3FILL2'       => { sites =>    2, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH3-N-0002'  },
  'TH3FILL1'       => { sites =>    1, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH3-N-0001'  },
  ######## UX7LS(12Grid) ########
  #### UX7LS(12Grid) standard decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TH1FILL1024GC3' => { sites => 1024, cap =>   1318.00, poly => 29.57,  leak =>  19.57, footp => 'UX7LSTH1-C-1024'  },
  'TH1FILL512GC3'  => { sites =>  512, cap =>    659.00, poly => 29.54,  leak =>  9.784, footp => 'UX7LSTH1-C-0512'  },
  'TH1FILL256GC3'  => { sites =>  256, cap =>    329.50, poly => 29.50,  leak =>  4.892, footp => 'UX7LSTH1-C-0256'  },
  'TH1FILL128GC3'  => { sites =>  128, cap =>    164.70, poly => 29.40,  leak =>  2.446, footp => 'UX7LSTH1-C-0128'  },
  'TH1FILL64GC3'   => { sites =>   64, cap =>     82.37, poly => 29.21,  leak =>  1.223, footp => 'UX7LSTH1-C-0064'  },
  'TH1FILL32GC3'   => { sites =>   32, cap =>     41.19, poly => 28.83,  leak =>  0.612, footp => 'UX7LSTH1-C-0032'  },
  'TH1FILL16GC3'   => { sites =>   16, cap =>     19.71, poly => 26.87,  leak =>  0.294, footp => 'UX7LSTH1-C-0016'  },
  'TH1FILL8GC3'    => { sites =>    8, cap =>      8.40, poly => 24.11,  leak =>  0.131, footp => 'UX7LSTH1-C-0008'  },
  #### UX7LS(12Grid) normal fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'TH1FILL1024'    => { sites => 1024, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH1-N-1024'  },
  'TH1FILL512'     => { sites =>  512, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH1-N-0512'  },
  'TH1FILL256'     => { sites =>  256, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH1-N-0256'  },
  'TH1FILL128'     => { sites =>  128, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH1-N-0128'  },
  'TH1FILL64'      => { sites =>   64, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH1-N-0064'  },
  'TH1FILL32'      => { sites =>   32, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH1-N-0032'  },
  'TH1FILL16'      => { sites =>   16, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH1-N-0016'  },
  'TH1FILL8'       => { sites =>    8, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH1-N-0008'  },
  'TH1FILL4'       => { sites =>    4, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH1-N-0004'  },
  'TH1FILL2'       => { sites =>    2, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH1-N-0002'  },
  'TH1FILL1'       => { sites =>    1, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX7LSTH1-N-0001'  },
);

} elsif ($tech_name =~ /UX8L/ || $tech_name =~ /UX8LD/ ) {
%CELL_TABLE = (
  ######## UX8L/UX8LD(9Grid) ########
  #### UX8L(9Grid) standard decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'LDL_FILL64GC1'  => { sites =>   64, cap =>     41.64, poly => 48.49,  leak =>  1.439, footp => 'UX8LLDL_-C-0064'  },
  'LDL_FILL32GC1'  => { sites =>   32, cap =>     20.82, poly => 56.85,  leak =>  0.865, footp => 'UX8LLDL_-C-0032'  },
  'LDL_FILL16GC1'  => { sites =>   16, cap =>      6.64, poly => 39.35,  leak =>  0.272, footp => 'UX8LLDL_-C-0016'  },
  'LDL_FILL8GC1'   => { sites =>    8, cap =>      1.35, poly => 27.97,  leak =>  0.031, footp => 'UX8LLDL_-C-0008'  },
  #### UX8L/UX8LD(9Grid) low leak decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'LDL_FILL64GC3'  => { sites =>   64, cap =>     31.70, poly => 48.50,  leak =>  0.609, footp => 'UX8LLDL_-C-0064'  },
  'LDL_FILL32GC3'  => { sites =>   32, cap =>     15.85, poly => 48.77,  leak =>  0.305, footp => 'UX8LLDL_-C-0032'  },
  'LDL_FILL16GC3'  => { sites =>   16, cap =>      5.13, poly => 39.83,  leak =>  0.115, footp => 'UX8LLDL_-C-0016'  },
  #### UX8L/UX8LD(9Grid) normal fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'LDL_FILL64'     => { sites =>   64, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX8LLDL_-N-0064'  },
  'LDL_FILL32'     => { sites =>   32, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX8LLDL_-N-0032'  },
  'LDL_FILL16'     => { sites =>   16, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX8LLDL_-N-0016'  },
  'LDL_FILL8'      => { sites =>    8, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX8LLDL_-N-0008'  },
  'LDL_FILL4'      => { sites =>    4, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX8LLDL_-N-0004'  },
  'LDL_FILL2'      => { sites =>    2, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX8LLDL_-N-0002'  },
  'LDL_FILL1'      => { sites =>    1, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX8LLDL_-N-0001'  },
  ######## UX8L/UX8LD(12Grid) ########
  #### UX8L/UX8LD(12Grid) standard decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'LPL_FILL64GC1'  => { sites =>   64, cap =>     66.54, poly => 56.79,  leak =>  2.842, footp => 'UX8LLPL_-C-0064'  },
  'LPL_FILL32GC1'  => { sites =>   32, cap =>     33.27, poly => 57.48,  leak =>  1.421, footp => 'UX8LLPL_-C-0032'  },
  'LPL_FILL16GC1'  => { sites =>   16, cap =>     10.57, poly => 39.93,  leak =>  0.447, footp => 'UX8LLPL_-C-0016'  },
  'LPL_FILL8GC1'   => { sites =>    8, cap =>      2.28, poly => 23.21,  leak =>  0.050, footp => 'UX8LLPL_-C-0008'  },
  #### UX8L/UX8LD(12Grid) low leak decoupling fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'LPL_FILL64GC3'  => { sites =>   64, cap =>     51.70, poly => 48.47,  leak =>  1.001, footp => 'UX8LLPL_-C-0064'  },
  'LPL_FILL32GC3'  => { sites =>   32, cap =>     25.85, poly => 49.07,  leak =>  0.501, footp => 'UX8LLPL_-C-0032'  },
  'LPL_FILL16GC3'  => { sites =>   16, cap =>      8.23, poly => 39.93,  leak =>  0.189, footp => 'UX8LLPL_-C-0016'  },
  #### UX8L/UX8LD(12Grid) normal fillcell ####
  # cell_name           num_of_sites   cap[fF]           poly_density[%] leak[nA]        footprint_name
  'LPL_FILL64'     => { sites =>   64, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX8LLPL_-N-0064'  },
  'LPL_FILL32'     => { sites =>   32, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX8LLPL_-N-0032'  },
  'LPL_FILL16'     => { sites =>   16, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX8LLPL_-N-0016'  },
  'LPL_FILL8'      => { sites =>    8, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX8LLPL_-N-0008'  },
  'LPL_FILL4'      => { sites =>    4, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX8LLPL_-N-0004'  },
  'LPL_FILL2'      => { sites =>    2, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX8LLPL_-N-0002'  },
  'LPL_FILL1'      => { sites =>    1, cap =>         0, poly =>  0.00,  leak =>      0, footp => 'UX8LLPL_-N-0001'  },
);
}

my @sorted_cell_list = sort {
  $CELL_TABLE{$b}->{sites} <=> $CELL_TABLE{$a}->{sites}
  || $CELL_TABLE{$b}->{cap} <=> $CELL_TABLE{$a}->{cap}
} keys %CELL_TABLE;
if ($ignore_normal_fillcell) {
  @sorted_cell_list = grep {$CELL_TABLE{$_}->{cap} > 0} @sorted_cell_list;
}

# Generate footprint table
my %footprint_table = ();
my $cell;
foreach $cell (@sorted_cell_list) {
  my $footprint = $CELL_TABLE{$cell}->{footp};
  if (defined($footprint) and $footprint ne '') {
    push(@{$footprint_table{$footprint}}, $cell);
  }
}

# Read journal and output report
my $found = 0;
my %current_cell_count = ();
while (<>) {
  my $line = $_;
  if ($line =~ /([^ ]+) : ([0-9]+) data found$/) {
    my $cell = $1;
    my $count = $2;
    if (exists $CELL_TABLE{$cell}) {
      $found++;
      $current_cell_count{$cell} = $count;
    }
  } elsif ($line =~ /#\/\/ ======== begin fillcell report .* ========$/) {
    %current_cell_count = ();
  } elsif ($line =~ /#\/\/ ======== end fillcell report \((.*)\) ========$/) {
    my $cap = report_cdfill($1);
    analyze_footprint($1) if ($cap >= 0);
    %current_cell_count = ();
  }
}
if ($found == 0) {
  print STDERR "No information found.\n";
  exit 1;
}
my $cap = report_cdfill('unknown');
analyze_footprint('unknown') if ($cap >= 0);
exit 0;


######## subroutines ########

sub get_option
{
  my $tech_flag=0 ;

  while ($#ARGV >= 0) {
    if ($ARGV[0] =~ "^-max-poly=\(.*\)") {
      $max_poly_dens = $1;
      if ($max_poly_dens <= 0) {
        print STDERR 'Invalid value: ', $max_poly_dens, "\n";
        show_usage();
      }
    } elsif ($ARGV[0] eq '-ignore-normal') {
      $ignore_normal_fillcell = 1;
    } elsif ($ARGV[0] eq '-verbose') {
      $verbose = 1;
    } elsif ($ARGV[0] eq '-tech') {
      $tech_name = $ARGV[1];
      $tech_flag = 1;

      if (  ($tech_name ne "CB90L") && ($tech_name ne "CB90M")
         && ($tech_name ne "CB90H") && ($tech_name ne "CB130M")
         && ($tech_name ne "CB65L") && ($tech_name ne "UX7LS")
         && ($tech_name ne "UX8L") && ($tech_name ne "UX8LD")) {
        print STDERR 'Invalid technology name: ', $tech_name, "\n";
        show_usage();
      }
      shift(@ARGV);
    } elsif ($ARGV[0] =~ /^-/) {
      print STDERR 'Invalid option: ', $ARGV[0], "\n";
      show_usage();
    } else {
      last;
    }
    shift(@ARGV);
  }

  # Technology check
  if ( $tech_flag != 1) {
       print STDERR 'Invalid technology name: ', $tech_name, "\n";
       show_usage();
  }
}

sub show_usage
{
  print STDERR "Usage:\n";
  print STDERR $program_name, " -tech <CB130M|CB90L|CB90M|CB90H|CB65L|UX7LS|UX8L|UX8LD> [-max-poly <density[%]>] [-ignore-normal] [-verbose] <journal>\n";
  exit 1;
}

sub report_cdfill
{
  my $phase = shift;

  return -1 if (scalar(keys %current_cell_count) == 0);
  if (@REPORT_PHASE_LIST > 0) {
    return -1 if (! grep {$_ eq $phase} @REPORT_PHASE_LIST);
  }
  print "======== report ($phase) ========\n\n";
  my ($total_cap, $poly_dens) = calculate_total_cap(1, %current_cell_count);
  return $total_cap;
}

sub analyze_footprint
{
  my $phase = shift;

  if (@ANALYSIS_PHASE_LIST > 0) {
    return -1 if (! grep {$_ eq $phase} @ANALYSIS_PHASE_LIST);
  }

  print "======== footprint analysis ($phase) ========\n\n";
  if (scalar(keys %current_cell_count) == 0) {
    print "No decoupling cells used.\n";
    return 0;
  }

  # Enumerate footprint candidates
  my %footprint_count = ();
  my $cell;
  foreach $cell (@sorted_cell_list) {
    my $footprint = $CELL_TABLE{$cell}->{footp};
    if (defined($footprint) and $footprint ne '') {
      if (defined($current_cell_count{$cell}) and $current_cell_count{$cell} > 0) {
        $footprint_count{$footprint} += $current_cell_count{$cell};
      }
    }
  }
  my @sorted_footprint_list = sort keys %footprint_count;

  # Show available cells
  print "==== available cells ====\n";
  my $combinations = 1;
  my $footprint;
  foreach $footprint (@sorted_footprint_list) {
    my $num = scalar(@{$footprint_table{$footprint}});
    if ($num > 1) {
      $combinations *= $num;
      print "footprint $footprint : $footprint_count{$footprint} cells :";
      foreach $cell (@{$footprint_table{$footprint}}) {
        print " $cell";
      }
      print "\n";
    }
  }
  print "Number of combinations = $combinations\n\n";
  if ($combinations <= 1) {
    print "No alternative combinations found.\n";
    return 0;
  }

  print "==== condition ====\n";
  print "Max poly density = $max_poly_dens [%]\n";
  print "Ignoring normal fillcells = ", ($ignore_normal_fillcell ? "yes\n" : "no\n");
  print "\n";

  # Check candidates
  my @combi = ();
  my $i;
  for ( $i = 0 ; $i < @sorted_footprint_list ; $i++) {
    push(@combi, 0);
  }
  my $best_density = 0;
  my $best_cap = 0;
  my %best_cell_count = ();
  for (my $candidate_num = 1; 1; $candidate_num++) {
    my %new_cell_count = ();
    my $i = 0;
    my $footprint;
    foreach $footprint (@sorted_footprint_list) {
      my $cell = $footprint_table{$footprint}[$combi[$i]];
      my $count = $footprint_count{$footprint};
      $new_cell_count{$cell} = $count;
      $i++;
    }
    print "==== candidate $candidate_num ====\n" if ($verbose);
    my ($total_cap, $poly_dens) = calculate_total_cap($verbose, %new_cell_count);
    if ($poly_dens > $best_density && $poly_dens <= $max_poly_dens) {
      $best_density = $poly_dens;
      $best_cap = $total_cap;
      %best_cell_count = %new_cell_count;
    }
    # Increment combi index
    $i = 0;
    $combi[$i]++;
    while ($combi[$i] >= scalar(@{$footprint_table{$sorted_footprint_list[$i]}})) {
      $combi[$i] = 0;
      $i++;
      $combi[$i]++;
      last if (! defined($sorted_footprint_list[$i]));
    }
    last if (! defined($sorted_footprint_list[$i]));
  }

  # Show the best
  print "==== the best combination ====\n";
  if ($best_density > 0) {
    my ($total_cap, $poly_dens) = calculate_total_cap(1, %best_cell_count);
    return $total_cap;
  } else {
    print "Not found!\n";
    return 0;
  }
}

sub calculate_total_cap
{
  my ($show, %cell_count) = @_;

  if ($show) {
    print "name                      count  num_sites         cap[fF] poly[%]   leak[nA]\n";
    print "-------------------- ---------- ---------- --------------- ------- ----------\n";
  }
  my ($total_count, $total_num_sites, $total_cap, $total_poly, $total_leak) = ( 0, 0, 0, 0, 0 );
  my $cell;
  foreach $cell (@sorted_cell_list) {
    if (defined($cell_count{$cell})) {
      my $count     = $cell_count{$cell};
      my $num_sites = $count * $CELL_TABLE{$cell}->{sites};
      my $cap       = $count * $CELL_TABLE{$cell}->{cap};
      my $poly_dens = ($num_sites > 0) ? $CELL_TABLE{$cell}->{poly} : 0;
      my $poly_area = $poly_dens * $num_sites;
      my $leak      = $count * $CELL_TABLE{$cell}->{leak};
      if ($show) {
        printf "%-20s %10d %10d %15.2f %7.2f %10d\n",
               $cell, $count, $num_sites, $cap, $poly_dens, $leak;
      }
      $total_count += $count;
      $total_num_sites += $num_sites;
      $total_cap += $cap;
      $total_poly += $poly_area;
      $total_leak += $leak;
    }
  }
  my $poly_dens = ($total_num_sites != 0) ? $total_poly / $total_num_sites : 0;
  if ($show) {
    print "-------------------- ---------- ---------- --------------- ------- ----------\n";
    printf "total                %10d %10d %15.2f %7.2f %10d\n\n",
           $total_count, $total_num_sites, $total_cap, $poly_dens, $total_leak;
  }
  return $total_cap, $poly_dens;
}

