# Script   : mmmc_hold_sdfba.tcl
# Version  : V1.3.4
# Function : "SAMPLE" SoC Encounter MMMC HoldFix with SDF
# Date     : June.25th,2009
#
# Ver.1.3.4 2009-06-24
#

set TECHNOLOGY UX8L

# ============================================================================ #
# --- Data preparation --- #
# ============================================================================ #
# --- MMMC Configration --- #
setenv ENABLE_MMMC_RC_FACTORS 1

set DESIGN              cpu8top
set OPC_PATH            /home/product/shuttle-pmf/el-shuttle/ndk/database/OPC_LIB/release_for_GIT_20091225
set inputConfig        ../../database/conf/VDEC_soce.conf
set inputVerilog       ../08_09_cdfill_repair/fill.v.gz
set inputDef           ../08_09_cdfill_repair/fill.def.gz
set inputLef           ../../database/lef/lef
set inputSDC           ../../database/sdc/cpu8top_clock_200MHz_sta.tcl
#set outputVerilog      ./mmmc_hold.v.gz
#set outputDef          ./mmmc_hold.def.gz
set ViewDefFile        ./file/view.mmmc
set DerateDefFile      ./file/timing_derate.mmmc
set dontuse            ../03_place/file/dont_use.tcl

set clkSpec            ../04_cts/file/cts_spec

set maxLayerNumber     7

if { $TECHNOLOGY == "CB65L" || $TECHNOLOGY == "UX7LS" } {
        set dbgUseSiteAsIGU 1
}

set I_LIB_MAX_WIDE1           ${OPC_PATH}/blib/UX8L/wide1_1.1V/liberty/max
set I_LIB_MAX_WIDE2           ${OPC_PATH}/blib/UX8L/wide2_1.1V/liberty/max
set I_LIB_MIN_WIDE1           ${OPC_PATH}/blib/UX8L/wide1_1.1V/liberty/min
set I_LIB_MIN_WIDE2           ${OPC_PATH}/blib/UX8L/wide2_1.1V/liberty/min

set WIDE1_MAX_LIB_LIST [ glob $I_LIB_MAX_WIDE1/*.lib ]
set WIDE2_MAX_LIB_LIST [ glob $I_LIB_MAX_WIDE2/*.lib ]
set WIDE1_MIN_LIB_LIST [ glob $I_LIB_MIN_WIDE1/*.lib ]
set WIDE2_MIN_LIB_LIST [ glob $I_LIB_MIN_WIDE2/*.lib ]

# --- View Definition File --- #
set VIEW_DEF_FILE       $ViewDefFile

# --- Analysis Corners --- #
set SETUP_ANALYSIS_CORNERS [list view-setup-wide2_max] 
set HOLD_ANALYSIS_CORNERS  [list view-hold-wide1_min \
                                 view-hold-wide2_min]

# --- Timing Derate Definition File --- #
set DERATE_DEF_FILE $DerateDefFile

# --- Input Sdf File --- #
set WIDE2_MAX_SDF ../sdf/wide2_1.1V/${DESIGN}.max.sdf.gz
set WIDE1_MIN_SDF ../sdf/wide1_1.1V/${DESIGN}.min.sdf.gz
set WIDE2_MIN_SDF ../sdf/wide2_1.1V/${DESIGN}.min.sdf.gz

# ============================================================================ #
# --- Data in --- #
# ============================================================================ #

setImportMode -keepEmptyModule 1
setDoAssign on -new
set dbgDefOutPhysicalPin 1
set dbgUpdateDefRows 0
set dbgGPSAutoCellFunction 1

### for general flow (from here) ###
loadConfig $inputConfig 0
set rda_Input(ui_netlist)          $inputVerilog
set rda_Input(ui_leffile)          $inputLef
set rda_Input(ui_timingcon_file)   $inputSDC
set rda_Input(ui_timelib,max)      $WIDE2_MAX_LIB_LIST
set rda_Input(ui_timelib,min)      $WIDE2_MIN_LIB_LIST
set rda_Input(ui_view_definition_file)  $VIEW_DEF_FILE
#set rda_Input(ui_buf_footprint)   $FP_BUF
#set rda_Input(ui_inv_footprint)   $FP_INV
#set rda_Input(ui_delay_footprint) $DELAY_BUF
commitConfig
if { [ info exists inputECOFile ] } { loadECO  $inputECOFile }
defIn $inputDef
### for general flow (to here) ###


setOpCond {}
setPrerouteAsObs {}
setTrialRouteMode -maxRouteLayer $maxLayerNumber -handlePreroute true

# ============================================================================ #
# --- Set Analysis View --- #
# ============================================================================ #

set_analysis_view -setup $SETUP_ANALYSIS_CORNERS \
                  -hold $HOLD_ANALYSIS_CORNERS

set timerFastCPPRInCTE 0
set_global timing_cppr_threshold_ps 1
set opgPROptEnableCompeteMode 0

# ============================================================================ #
# --- CRPR Setting ---
# ============================================================================ #
source $DERATE_DEF_FILE

# ============================================================================ #
# --- Set CTS Spec File --- #
# ============================================================================ #
specifyClockTree -file $clkSpec
#changeClockStatus -all -fixedNetWires -fixedBuffers -useClock

# ============================================================================ #
# --- Run STA --- #
# ============================================================================ #
setCteReport
set_interactive_constraint_modes [all_constraint_modes -active]
set_propagated_clock [all_clocks]
set_clock_uncertainty -setup 0.30 [all_clocks]
set_clock_uncertainty -hold  0.08 [all_clocks]
timeDesign -postRoute -outDir preopt_hold_rpt -numPaths 1000 -hold
timeDesign -postRoute -outDir preopt_setup_rpt -numPaths 1000

# ============================================================================ #
# --- HoldFix --- #
# ============================================================================ #
#set opgTimeDesignMmmcExpandedViews 1

setOptMode \
        -restruct false \
        -fixDrc false \
        -optimizeFF false \
        -downsizeInst false \
        -fixHoldAllowSetupTnsDegrade false \
        -fixHoldSearchRadius 80 \
        -setupTargetSlack 0.0 \
        -holdTargetSlack 0.00 \
        -fixHoldForceAtPathEnd true

## setPlaceMode -allowBorderPinAbut true

setAnalysisMode -checkType hold -analysisType onChipVariation
set opgSDFIncrementalMode 1
set optIgnoreOCVMode 0
set dbgDcRdRNetFractionThreshold 1.0

# turn on post route legal location resizing
# set resizing search radius to 5 rows
setOptMode -legalLocGS true
setOptMode -searchRadiusGS 5


read_sdf -view view-setup-wide2_max $WIDE2_MAX_SDF 
read_sdf -view view-hold-wide1_min  $WIDE1_MIN_SDF
read_sdf -view view-hold-wide2_min  $WIDE2_MIN_SDF

#timeDesign -reportOnly -outDir preopt_Setup_rpt -numPaths 1000
#timeDesign -reportOnly -hold -outDir preopt_Hold_rpt -numPaths 1000

# If Setup is degrade by HoldFix, please set following options. 
setOptMode -fixHoldAllowSetupTnsDegrade true
set opgFixHoldAggressive 16

source $dontuse

#optDesign -postRoute -noECORoute -outDir postRouteHoldFix.rpt -hold -useSDF

timeDesign -reportOnly -outDir postopt_Setup_rpt
timeDesign -reportOnly -hold -outDir postopt_Hold_rpt 

# ============================================================================ #
# --- SaveDesign --- #
# ============================================================================ #
#changeClockStatus -all -noFixedNetWires -fixedBuffers -useClock
#
#if { $TECHNOLOGY == "CB65L" || $TECHNOLOGY == "UX7LS" || $TECHNOLOGY == "UX8L" } {
#        set dbgLefDefOutVersion 5.6
#} else {
#        set dbgLefDefOutVersion 5.5
#}
#
#if { $TECHNOLOGY == "UX8L" } {
#        globalNetConnect VDD -type pgpin -pin VDD
#        globalNetConnect VSS -type pgpin -pin VSS
#} else {
#        globalNetConnect VDD -type pgpin -pin VDD
#        globalNetConnect GND -type pgpin -pin GND
#}
#
#if { $TECHNOLOGY == "UX7LS" } {
#        globalNetConnect GND -type tielo
#        globalNetConnect VDD -type tiehi
#}
#
#if { $TECHNOLOGY == "UX8L" } {
#        globalNetConnect VSS -type tielo
#        globalNetConnect VDD -type tiehi
#}
#
#deleteDanglingPort

### for general flow (from here) ###
#defOut -floorplan -placement -noCutRow -routing $outputDef
### for general flow (to here) ###

### for flat flow (from here) ###
#defOut -verilog_from_def_netlist_flow -floorplan -placement -noCutRow -routing $outputDef
### for flat flow (to here) ###

#if { [ info exists inputECOFile ] } {
#  loadECO  $inputECOFile_DEL
#  source   $inputDelModule
#  saveNetlist -excludeCellInst $Phy_Only_Cell $outputVerilog
#} else {
#  saveNetlist $outputVerilog
#}

exit

