# Script   : hold_repair.tcl (from init_edit.tcl)
# Version  : Ver4.0.0
# Function : "SAMPLE" Script for SoC Encounter Data Input for Edit
# Date     : Mar.30,2009
#
#    Ver4.0.0 Mar-27-2009
#

set TECHNOLOGY UX8L
set SOCE_VER 8.1
set ENCOUNTER_VERSION 8.1

# ============================================================================ #
# --- Data preparation --- #
# ============================================================================ #

set inputConfig		../../database/conf/VDEC_soce.conf
set inputVerilog	../15_timing_hold/mmmc_hold.v.gz
set inputDef		../15_timing_hold/mmmc_hold.def.gz
set inputLef            ../../database/lef/lef
set inputSDC            ../../database/sdc/cpu8top_clock_200MHz.tcl
set outputVerilog	./hold_repair.v.gz
set outputDef		./hold_repair.def.gz
#set IbpoNet             ./file/err_net

set maxLayerNumber	7

#set inputECOFile        <input ECO file>

# ============================================================================ #
# --- Initial Setting --- #
# ============================================================================ #

if { $TECHNOLOGY == "CB12" || $TECHNOLOGY == "CB130M"|| $TECHNOLOGY == "CMOS12M" } {
        set DBU 100
} else {
        set DBU 1000
}

set numberOfPaths 100

set dbgUpdateDefRows 0
set dbgGPSAutoCellFunction 1

set dbgDefOutUnits $DBU
setImportMode -minDbuPerMicron $DBU
set preRouteSkipApplyGlobalNets 1
setVar dbgAllowPowerDomainMinGapZero 1
set sprUseRowOrient 1

set dbgIPOMaxBufDetourDist 100
set dpgBufInsMinTotCap 0

# ============================================================================ #
# --- Data in --- #
# ============================================================================ #
setImportMode -keepEmptyModule 1
setDoAssign on -new

### for general flow (from here) ###
loadConfig				$inputConfig 0
set rda_Input(ui_netlist)		$inputVerilog
set rda_Input(ui_leffile)               $inputLef
set rda_Input(ui_timingcon_file)        $inputSDC
commitConfig

if { [ info exists inputECOFile ] } { loadECO  $inputECOFile }

defIn	$inputDef
### for general flow (to here) ###

setCteReport
setOpCond {}

# ============================================================================ #
# --- Setting After Importing Design --- #
# ============================================================================ #

set designName [dbgDesignName]

if { $TECHNOLOGY == "CB65L" || $TECHNOLOGY == "UX7LS" } {
        set dbgLefDefOutVersion 5.6
} else {
        set dbgLefDefOutVersion 5.5
}

# ============================================================================ #
# insert repeater ( signal net )
# ============================================================================ #

#insertRepeater -postRoute -check -rule file/ibpo.rule -selNet $IbpoNet
ecoAddRepeater -net core/clk_blk/N0 -cell LDN_BUF_10
ecoAddRepeater -net core/clk_blk/N0 -cell LDN_BUF_10
ecoAddRepeater -net core/clk_blk/N0 -cell LDN_BUF_10

ecoAddRepeater -net core/cpu/cpu_rf/N17 -cell LDN_BUF_4
ecoAddRepeater -net core/n9 -cell LDN_BUF_10

# ============================================================================ #
# --- SaveDesign --- #
# ============================================================================ #
if { $TECHNOLOGY == "CB65L" || $TECHNOLOGY == "UX7LS" || $TECHNOLOGY == "UX8L" } {
        set dbgLefDefOutVersion 5.6
} else {
        set dbgLefDefOutVersion 5.5
}

deleteDanglingPort

if { $TECHNOLOGY == "UX8L" } {
        globalNetConnect VDD -type pgpin -pin VDD
        globalNetConnect VSS -type pgpin -pin VSS
} else {
        globalNetConnect VDD -type pgpin -pin VDD
        globalNetConnect GND -type pgpin -pin GND
}

if { $TECHNOLOGY == "UX7LS" } {
        globalNetConnect GND -type tielo
        globalNetConnect VDD -type tiehi
}

if { $TECHNOLOGY == "UX8L" } {
        globalNetConnect VSS -type tielo
        globalNetConnect VDD -type tiehi
}

defOut -floorplan -placement -netlist \
        -noCutRow -routing $outputDef

if { [ info exists inputECOFile ] } {
  loadECO  $inputECOFile_DEL
  source   $inputDelModule
  saveNetlist -excludeCellInst $Phy_Only_Cell $outputVerilog
} else {
  saveNetlist $outputVerilog
}

# ============================================================================ #
# --- Exit --- #
# ============================================================================ #
exit
# END
